<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Controllers
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: DesignController.php 1545 2010-09-23 08:41:52Z ms $
 */
class Sicore_DesignController extends Sicore_Controllers_Simploo
{
    const ID_PREFIX   = 'si_tree_design_';  
    
    public function init()
    {
        $this->_helper->contextSwitch()->addActionContext('tree', 'json');
        $this->_helper->contextSwitch()->addActionContext('load', 'json');  
        $this->_helper->contextSwitch()->addActionContext('choosertree', 'json');  
        parent::init();
    }
    
    public function treeAction()
    {
        $mPages = new Sicore_Models_Gui_Menu_Design();
        $mPages->appendFilesToView();
        $mPages->render(); 
    }
    
    public function loadAction()
    {
        $parent    = $this->getRequest()->getParam('parent');
        $isDraft   = (bool)$this->getRequest()->getParam('draft',  false); 
        
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        
        if(empty($parent)){
            $data   = array();
            $data[] = array('attributes' => array('id'              => self::ID_PREFIX.'local',
                                                  'folder'          => 'local', 
                                                  'parent'          => '',
                                                  'title'           => $this->view->translate('DESIGN_TEMPLATES'),
                                                  'metadata'        => array("draggable" => true), 
                                                  'isDraft'         => intval($isDraft),  
                                                  'canEdit'         => 0,
                                                  'canShow'         => 0,
                                                  'canDelete'       => 0
                                                ),         
                              'state'      => 'closed',
                              'data'       => array('title' => $this->view->translate('DESIGN_TEMPLATES'), 
                                                    'icon'  => 'icon_container.gif'
                                                    ) 
                              );
            if($this->config_base->onlinepool->designs){
                $data[] = array('attributes' => array('id'              => self::ID_PREFIX.'pool_',
                                                      'uid'             => 'pool_', 
                                                      'folder'          => 'pool_',
                                                      'sitype'          => 'folder',  
                                                      'parent'          => '',
                                                      'title'           => $this->view->translate('DESIGN_TEMPLATES_POOL'),
                                                      'metadata'        => array("draggable" => false), 
                                                      'isDraft'         => intval($isDraft),  
                                                      'canEdit'         => 0,
                                                      'canShow'         => 1,
                                                      'canDelete'       => 0
                                                    ),         
                                  'state'      => 'closed',
                                  'data'       => array('title' => $this->view->translate('DESIGN_TEMPLATES_POOL'), 
                                                        'icon'  => 'icon_pool.gif'
                                                        ) 
                                  );  
            }
        }elseif($parent == 'local'){
            $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
            $templates  = $mTemplates->getAll();
            
            $editable  = in_array('Sicore_Models_Gui_Menu_Design', $this->user->gui_menu_enabled->toArray()) || $this->user->isadmin->value();
            $data      = array();
            foreach($templates as $template){                
                if($config_frontend->layout == $template->getFolder()){
                    $icon        = 'icon_template_default.gif';
                    $prefix      = ' ('.$this->view->translate('DESIGN_TEMPLATE_STANDARD').')';
                    $isStandard  = 1;
                }else{
                     $icon       = 'icon_template.gif';
                     $prefix     = '';
                     $isStandard = 0;
                } 
                $data[] = array('attributes' => array('id'              => self::ID_PREFIX.$template->getFolder(),
                                                      'folder'          => $template->getFolder(),
                                                      'uid'             => $template->getFolder(), 
                                                      'parent'          => 'local',  
                                                      'title'           => $template->getName().$prefix,
                                                      'metadata'        => array("draggable" => true),
                                                      'editAction'      => 'localedit',
                                                      'showAction'      => 'localshow',    
                                                      'isDraft'         => intval($isDraft),       
                                                      'canEdit'         => intval($editable && $isDraft),
                                                      'isStandard'      => $isStandard,
                                                      'canShow'         => 1,
                                                      'canDelete'       => intval($template->canDelete()),
                                                      'object'          => get_class($template) 
                                                    ),         
                                  'state'      => 'closed',
                                  'data'       => array('title' => $template->getName(), 
                                                        'icon'  => $icon
                                                        ) 
                                  );  
            }
        }else{
            $parent = $parent == 'pool_' ? '' : substr($parent, 5);
            $mTemplates = new Sicore_Models_Server_Http_Templates($this->config_base->app->updates);
            $mTemplates->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);
            $categories  = $mTemplates->getCategories($parent);
            if(!$categories['error']){
                foreach($categories as $category){
                    $data[] = array('attributes' => array('id'              => self::ID_PREFIX.'pool_'.$category['uid'],
                                                          'uid'             => 'pool_'.$category['uid'], 
                                                          'folder'          => 'pool_'.$category['uid'], 
                                                          'parent'          => 'pool_'.$category['uid_parent'],
                                                          'title'           => $category['title'],
                                                          'sitype'          => 'folder',
                                                          'metadata'        => array("draggable" => false), 
                                                          'isDraft'         => intval($isDraft),  
                                                          'canEdit'         => 0,
                                                          'canShow'         => 1,
                                                          'canDelete'       => 0
                                                        ),         
                                      'state'      => 'closed',
                                      'data'       => array('title' => $category['title'], 
                                                            'icon'  => 'icon_pool2.gif'
                                                            ) 
                                      ); 
                } 
            }else{
                Sicore_Models_Logger::error($categories['error']);
                $data = array();
            }
        }   
        if($this->_helper->contextSwitch()->getCurrentContext() == 'json'){
            echo Zend_Json::encode($data);
            die();
        } 
    }
    
    public function choosertreeAction()
    {
        $parent    = $this->getRequest()->getParam('uid');
        
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        
        if(empty($parent)){
            $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
            $templates  = $mTemplates->getAll();
            
            $editable  = in_array('Sicore_Models_Gui_Menu_Design', $this->user->gui_menu_enabled->toArray()) || $this->user->isadmin->value();
            $data      = array();
            foreach($templates as $template){                
                if($config_frontend->layout == $template->getFolder()){
                    $icon        = 'icon_template_default.gif';
                    $prefix      = ' ('.$this->view->translate('DESIGN_TEMPLATE_STANDARD').')';
                    $isStandard  = 1;
                }else{
                     $icon       = 'icon_template.gif';
                     $prefix     = '';
                     $isStandard = 0;
                } 
                $data[] = array('attributes' => array('id'              => self::ID_PREFIX.$template->getFolder(),
                                                      'folder'          => $template->getFolder(),
                                                      'uid'             => $template->getFolder(), 
                                                      'parent'          => 'local',  
                                                      'title'           => $template->getName().$prefix,
                                                      'metadata'        => array("draggable" => true),
                                                      'editAction'      => 'localedit',
                                                      'showAction'      => 'localshow',    
                                                      'isDraft'         => intval($isDraft),       
                                                      'canEdit'         => intval($editable && $isDraft),
                                                      'isStandard'      => $isStandard,
                                                      'canShow'         => 1,
                                                      'canDelete'       => intval($template->canDelete()),
                                                      'object'          => get_class($template) 
                                                    ),         
                                  'state'      => 'opened',
                                  'data'       => array('title' => $template->getName(), 
                                                        'icon'  => $icon
                                                        ) 
                                  );  
            }
            
        } 
        if($this->_helper->contextSwitch()->getCurrentContext() == 'json'){
            echo Zend_Json::encode($data);
            die();
        } 
    }
    
    public function choosercontentAction()
    {
        $uid = $this->getRequest()->getParam('uid');
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        
        $data     = array();
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
                
        $this->view->template = $mTemplates->get($uid);  

        $content = $this->view->render('chooser/design.phtml'); 
        
        $data    = array('status'  => true,
                         'content' => $content
                        );
                        
        die(Zend_Json::encode($data));
    }
    
    public function editAction()
    {
        $translate  = Sicore_Models_Registry::getTranslate(); 
        
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        
        $data       = array();
        $folder     = $this->getRequest()->getParam('folder');
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
        
        $template   = $mTemplates->get($folder); 
        $isStandard = $config_frontend->layout == $template->getFolder();
        $form       = new Sicore_Models_Gui_Form_Design_Template($template, $this->user, $isStandard);
        
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);  
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path;
        $form->setAction($path.'sicore/design/save');
        
        $data['labelShow']     = $translate->_('GENERAL_SHOW2');
        $data['title']         = $this->view->translate('SETTINGS_UPDATES');
        $data['content']       = $form->render();
        $data['scripts']       = $this->view->jQuery()->getOnLoadActions();
        $data['scripts'][]     = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';   

        echo Zend_Json::encode($data);
        die(); 
    }
    
    
    public function saveAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        $folder    = $this->getRequest()->getParam('folder'); 
        
        if($this->getRequest()->isPost()){
            $formData   = $this->getRequest()->getPost(); 
            $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
            $template   = $mTemplates->get($folder); 
            $isStandard = $config_frontend->layout == $template->getFolder();
            $form       = new Sicore_Models_Gui_Form_Design_Template($template, $this->user, $isStandard);
            if(!$form->isValid($formData)){
                $errors = $form->getBackendErrors();
                $data['saved']     = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                $data['saved']     = true; 
                $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
            } 
        }
        
        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function activateAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $folder = $this->getRequest()->getParam('folder'); 
        if(file_exists(APPLICATION_PATH . '/config/custom/frontend.ini.php')){
            $config = new Zend_Config_Ini(APPLICATION_PATH . '/config/custom/frontend.ini.php', null, array('skipExtends' => true, 'allowModifications' => true));
            $data   = $config->toArray();
        }else{
            $data   = array();
        }
        $data['layout'] = $folder;
        
        try{
            $writer = new Simploo_Config_Writer_Ini();
            $writer->write(APPLICATION_PATH . '/config/custom/frontend.ini.php', new Zend_Config($data, true));
            $data['saved']     = true; 
            $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
            $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
        }catch(Zend_Config_Exception $e){
            Sicore_Models_Logger::exception($e);
            $data['saved']     = false;
            $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
            $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
        } 
    }
    
    public function removeAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $folder = $this->getRequest()->getParam('folder');            
        $path = Sicore_Models_Helper_FSO::joinPathAbs();
       
        try{
            if($this->config_base->ftp->enable){
                $ftp = new Sicore_Models_Server_Ftp($this->config_base->ftp->server,
                                                    $this->config_base->ftp->user,
                                                    $this->config_base->ftp->password,
                                                    $this->config_base->ftp->remotedir,
                                                    $this->config_base->ftp->pasv,
                                                    $this->config_base->ftp->ssl);
            }else{
                $ftp = null;
            }
            
            if(is_dir($path.'templates/frontend/'.$folder)){
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend', 755, $ftp);
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend/'.$folder, 755, $ftp); 
            }else{
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend', 755, $ftp); 
            }
            if(!is_writable($path.'templates/frontend/'.$folder) || !is_writable($path.'templates/frontend')) throw new Exception("Folder '$path/templates/frontend' not writable.");
            Sicore_Models_Helper_FSO::deleteDir($path.'templates/frontend/'.$folder);
            $data['saved']     = true; 
            $data['msg_title'] = $translate->_('LL_TEMPLATE_REMOVE_MSG_TITLE');
            $data['msg_text']  = $translate->_('LL_TEMPLATE_REMOVE_MSG_TEXT');
        }catch(Zend_Config_Exception $e){
            Sicore_Models_Logger::exception($e);
            $data['saved']     = false;
            $data['msg_title'] = $translate->_('LL_TEMPLATE_REMOVE_ERROR_TITLE');
            $data['msg_text']  = $translate->_('LL_TEMPLATE_REMOVE_ERROR_TEXT');
        }
        
        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function showAction()
    {
        $translate = Sicore_Models_Registry::getTranslate(); 
        $folder = $this->getRequest()->getParam('folder');  
        
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
        $template   = $mTemplates->get($folder); 
        
        $this->view->params  = array('draft' => 0, 'template' => $folder, 'nocache' => time());
        $data['labelInfos']    = $translate->_('GENERAL_INFOS');
        $data['labelActivate'] = $translate->_('SICORE_MODELS_NODE_PAGE_TEMPLATE_ACTIVATE');
        $data['labelColors']   = $translate->_('DESIGN_CHANGE_COLORS');  
        $data['labelImages']   = $translate->_('DESIGN_CHANGE_IMAGES');
        $data['labelRefresh']  = $translate->_('GENERAL_REFRESH');    
        $data['title']         = $folder;
        $data['content']       = $this->view->render('design/preview.phtml'); 
        if($template->isColorWheelAvailable()){
            $data['colors']    = $template->getColorsAsArray();
        }else{
            $data['colors']    = null;
        }
        $data['images'] = $template->isImageEditorAvailable(); 
        echo Zend_Json::encode($data);
        die();
    }
    
    public function savecolorsAction()
    {
        $translate = Sicore_Models_Registry::getTranslate();
        
        $folder = $this->getRequest()->getParam('folder');
        $params = $this->getRequest()->getParams();
        
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
        $template   = $mTemplates->get($folder); 
        $colors     = $template->getColorsAsArray(); 
        $cssContent = '';
        foreach($colors as $number => $entry){
            foreach($entry as $key => $style){
                if($style['brighten']){
                    $color = '#'.strtoupper($params["color_${number}_b"]);
                }else{
                    $color = '#'.strtoupper($params["color_$number"]);
                }
                $cssContent .= $style['selector']."{\n";
                $cssContent .= "\t".$style['property']." : ".$color.";\n}\n";
            }   
        }
        
        try{
            $file = APPLICATION_PATH.'/templates/frontend/'.$folder.'/'.$template->getConfig()->colors->css;
            file_put_contents($file, $cssContent);
            $data['saved']     = true; 
            $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
            $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e);
            $data['saved']     = false;  
            $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
            $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function imageseditorAction()
    {
        $translate  = Sicore_Models_Registry::getTranslate(); 
        
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        
        $data       = array();
        $folder     = $this->getRequest()->getParam('folder');
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
        
        $template   = $mTemplates->get($folder); 
        $form       = new Sicore_Models_Gui_Form_Design_Images($template, $this->user);
        
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path); 
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path;  
        $form->setAction($path.'sicore/design/imagessave');
        
        $data['labelSave']     = $translate->_('GENERAL_SAVE');
        $data['labelCancel']   = $translate->_('GENERAL_CANCEL');
        $data['title']         = $this->view->translate('DESIGN_CHANGE_IMAGES');
        $data['content']       = $form->render();
        $data['scripts']       = $this->view->jQuery()->getOnLoadActions();
        $data['scripts'][]     = 'var oldvals1 = $(\'#si_edit_main\').trackChanges({excludeClass: \'si_notrack\', changeListName: "si_edit_main_changed", events: "change blur keypress keydown click"});';   

        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function imagessaveAction()
    {
        $translate  = Sicore_Models_Registry::getTranslate(); 
        
        $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        
        $data       = array();
        $folder     = $this->getRequest()->getParam('folder');
        $mTemplates = new Sicore_Models_Server_Templates(APPLICATION_PATH.'/templates/frontend');
        $template   = $mTemplates->get($folder);
         
        if($this->getRequest()->isPost()){
            $formData= $this->getRequest()->getPost(); 
            $form = new Sicore_Models_Gui_Form_Design_Images($template, $this->user);
            if(!$form->isValid($formData)){
                $errors = $form->getBackendErrors();
                $data['saved']     = false;
                $data['errors']    = $errors['errors'];
                $data['fields']    = $errors['fields'];
                $data['subforms']  = $errors['subformIndexs'];
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT'); 
            }else{
                $images = $template->getImagesAsArray();
                foreach($images as $key => $image){
                    $destImagePath = Sicore_Models_Helper_FSO::joinPathAbs('templates/frontend', $folder).$image['path'];
                    if(!empty($formData["delete_$key"])){  
                        Sicore_Models_Helper_FSO::deleteFile($destImagePath);    
                    }
                    if(!empty($formData[$key])){
                        $mImages      = new Sicore_Models_Server_Db_Files();
                        $srcImage     = $mImages->get($formData[$key]);    
                        $srcImagePath = $srcImage->getRealPath();
                        
                        $mImage = new Sicore_Models_Image(); 
                        $mImage->convertFromConfig($srcImagePath, $destImagePath, $image['size']);
                    }  
                }  
                try{
                    $data['saved']     = true; 
                    $data['msg_title'] = $translate->_('GENERAL_FORM_SAVED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_SAVED_TEXT');
                }catch(Zend_Config_Exception $e){
                    Sicore_Models_Logger::exception($e);
                    $data['saved']     = false;
                    $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                    $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_CONFIGWRITE');
                }
                
            } 
        }
        
        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function pooltemplatesAction()
    {
        $uid     = $this->getRequest()->getparam('uid'); 
        $parent  = substr($uid, 5); 
        
        $mTemplates = new Sicore_Models_Server_Http_Templates($this->config_base->app->updates);
        $mTemplates->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);

        $templates = $mTemplates->getByFolder($parent, $this->config_base->app->release);
        $folders   = $mTemplates->getCategories($parent);
        $folder    = $mTemplates->getCategory($parent);
        
        $view->templates   = $templates['error']  ? array() : $templates; 
        $view->folders     = $folders['error']    ? array() : $folders; 
        $view->folder      = $folder; 

        $view->size        = $this->config_base->gfx->sizes->thumb;
        $this->view->data  = $view;
        
        $data['title']       = $view->folder['title'] ? $view->folder['title'] : $this->view->translate('DESIGN_TEMPLATES_POOL'); 
        $data['content']     = $this->view->render('design/templates.phtml');
        
        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function pooltemplateAction()
    {
        $uid  = $this->getRequest()->getparam('uid'); 
        $uid  = substr($uid, 5); 
        
        $mTemplates = new Sicore_Models_Server_Http_Templates($this->config_base->app->updates);
        $mTemplates->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);
        
        $template = $mTemplates->getByUid($uid); 
        if(!$template['error']){ 
            $this->view->pool      = $template;
            $data['status']        = (bool)$this->view->pool;
            $data['labelDownload'] = $this->view->translate('GENERAL_DOWNLOAD'); 
            $data['title']         = $template['title'];
            $data['content']       = $this->view->render('design/template.phtml'); 
        }else{
            Sicore_Models_Logger::error($template['error']);   
            $data['error'] = true; 
        }
        echo Zend_Json::encode($data);
        die(); 
    }
    
    public function downloadlicenceAction()
    {
        $uid = $this->getRequest()->getparam('uid');
        $uid = substr($uid, 5); 
        $mTemplates = new Sicore_Models_Server_Http_Templates($this->config_base->app->updates);
        $mTemplates->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);
        
        $template = $mTemplates->getByUid($uid);
        if(!$template['error']){ 
            $data['labelDownload'] = $this->view->translate('GENERAL_DOWNLOAD_ACCEPT');
            $data['labelCancel']   = $this->view->translate('GENERAL_CANCEL');    
            $data['content']       = $mTemplates->getLicence($uid); 
            $data['title']         = $template['title'];   
        }else{
            Sicore_Models_Logger::error($template['error']);  
            $data['error'] = true;
        }
        echo Zend_Json::encode($data);
        die();   
        
    }
    
    public function downloadtemplateAction()
    {
        $uid = $this->getRequest()->getparam('uid');
        $uid  = substr($uid, 5); 
        
        try{
            $mTemplatePool = new Sicore_Models_Server_Http_Templates($this->config_base->app->updates);
            $mTemplatePool->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);
            $tempFile = $mTemplatePool->download($uid);
            $template = $mTemplatePool->getByUid($uid);
            if(!$tempFile || $template['error']) throw new Exception('Template Download failed');  
            
            $folder = str_replace('.sip', '', $template['filename']);
            $path   = Sicore_Models_Helper_FSO::joinPathAbs();  
            
            if($this->config_base->ftp->enable){
                $ftp = new Sicore_Models_Server_Ftp($this->config_base->ftp->server,
                                                    $this->config_base->ftp->user,
                                                    $this->config_base->ftp->password,
                                                    $this->config_base->ftp->remotedir,
                                                    $this->config_base->ftp->pasv,
                                                    $this->config_base->ftp->ssl);
            }else{
                $ftp = null;
            }
            
            if(is_dir($path.'templates/frontend/'.$folder)){
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend', 755, $ftp);
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend/'.$folder, 755, $ftp); 
                Sicore_Models_Helper_FSO::deleteDir($path.'templates/frontend/'.$folder);
            }else{
                Sicore_Models_Helper_FSO::setFilePerm($path.'templates/frontend', 755, $ftp); 
            }
            
            $mPack = new Simploo_Pack();
            if(!is_writable($path.'templates/frontend/')) throw new Exception("Folder '$path/templates/frontend' not writable.");
            if(!Sicore_Models_Helper_FSO::mkdir($path.'templates/frontend/'.$folder)) throw new Exception("Folder '$path/templates/frontend/$folder' not created.");
            $mPack->extract($tempFile, $path.'templates/frontend/'.$folder);
            if(!file_exists($path.'templates/frontend/'.$folder.'/config.ini.php')) throw new Exception('Template Download failed: not extracted'); 
            @unlink($tempFile);
        }catch(Exception $e){
            @unlink($tempFile);
            Sicore_Models_Logger::exception($e);
            $data['status']    = false;
            $data['msg_title'] = $this->view->translate('LL_POOL_DOWNLOAD_ERROR_TITLE');
            $data['msg_text']  = $this->view->translate('LL_POOL_DOWNLOAD_ERROR_TEXT');
            echo Zend_Json::encode($data);
            die();
        }    
        
        $data['status']    = true;
        $data['msg_title'] = $this->view->translate('LL_POOL_DOWNLOAD_MESSAGE_TITLE');
        $data['msg_text']  = $this->view->translate('LL_POOL_DOWNLOAD_MESSAGE_TEXT');
        echo Zend_Json::encode($data);
        die();; 
    }
}