<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Controllers
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: MediaController.php 1549 2010-10-04 08:33:50Z ms $
 */
class Sicore_MediaController extends Sicore_Controllers_Simploo
{
    const ID_PREFIX_FOLDER = 'si_tree_fo_';
    const ID_PREFIX_FILE   = 'si_tree_fi_';
    
    public function init()
    {
        $this->_helper->contextSwitch()->addActionContext('tree',      'json');
        $this->_helper->contextSwitch()->addActionContext('load',      'json'); 
        $this->_helper->contextSwitch()->addActionContext('dropimage', 'json'); 
        $this->_helper->contextSwitch()->addActionContext('youtube',   'json'); 
        $this->_helper->contextSwitch()->addActionContext('choosertree',   'json');   
        $this->_helper->contextSwitch()->addActionContext('choosercontent',   'json');   
        parent::init();
    }
    
    public function treeAction()
    {
        $mPages = new Sicore_Models_Gui_Menu_Media();
        $mPages->appendFilesToView();
        $mPages->render(); 
    }
    
    public function loadAction()
    {
        
        $parentUid = $this->getRequest()->getparam('uid');
        
        $isDraft = (bool)$this->getRequest()->getParam('draft',  false); 
        if(substr($parentUid, 0, 5) == 'pool_'){
            $parent        = substr($parentUid, 5);
            $mImages = new Sicore_Models_Server_Http_Images($this->config_base->app->updates);
            $mImages->setCredentials($this->config_base->app->number, $this->config_base->customer->secret);
            $categories  = $mImages->getCategories($parent);
            if(!$categories['error']){     
                foreach($categories as $category){ 
                    $data[] = array('attributes' => array('id'         => self::ID_PREFIX_FOLDER.'pool_'.$category['uid'],
                                                          'uid'        => 'pool_'.$category['uid'], 
                                                          'parent'     => $category['uid_parent'], 
                                                          'title'      => $category['title'],
                                                          'folder'     => $category['folder'], 
                                                          'metadata'   => array("draggable" => false),
                                                          'sitype'     => 'folder',
                                                          'isDraft'    => intval($isDraft), 
                                                          'metadata'   => array("draggable" => true),
                                                          'object'     => 'pool',
                                                          'canRename'  => 0,
                                                          'canEdit'    => 0,
                                                          'canDelete'  => 0,
                                                          'canAddFolder'    => 0,
                                                          'canAddSubFolder' => 0,
                                                          'canUpload'       => 0,
                                                          'preview'         => false 
                                                        ),         
                                  'state'      => 'closed',
                                  'data'       => array('title' => $category['title'], 
                                                        'icon'  => 'icon_folder.gif'
                                                        ) 
                                  ); 
                } 
            }else{
                Sicore_Models_Logger::error($categories['error']);
                $data = array();
            }   
        }else{
            $mFolders = new Sicore_Models_Server_Db_Folders();
            $folders = $mFolders->getByParentUid($parentUid);
                    
            $data = array();
            
            $editable = in_array('Sicore_Models_Gui_Menu_Pages', $this->user->gui_menu_enabled->toArray()) || $this->user->isadmin->value(); 
            
            foreach($folders as $folder){
                if(intval($isDraft)){
                    $draggable = true;
                }else{
                    $draggable = false;
                }
                 
                $data[] = array('attributes' => array('id'         => self::ID_PREFIX_FOLDER.$folder->uid->value(),
                                                      'uid'        => $folder->uid->value(), 
                                                      'parent'     => $folder->uid_parent->value(),
                                                      'title'      => '['.$folder->uid->value().'] '.$folder->name->value(),
                                                      'metadata'   => array("draggable" => true),
                                                      'sitype'     => 'folder',
                                                      'isDraft'    => intval($isDraft), 
                                                      'metadata'   => array("draggable" => $draggable),
                                                      'object'     => get_class($folder),
                                                      'canRename'  => intval($isDraft),
                                                      'canEdit'    => intval($isDraft),
                                                      'canDelete'  => intval($isDraft && $folder->level->value()),
                                                      'canAddFolder'    => intval($isDraft && $folder->level->value()),
                                                      'canAddSubFolder' => intval($isDraft),
                                                      'canUpload'       => intval($isDraft),
                                                      'preview'         => false 
                                                    ),         
                              'state'      => 'closed',
                              'data'       => array('title' => $folder->name->value(), 
                                                    'icon'  => 'icon_folder.gif'
                                                    ) 
                              );   
            }
            
            $mFiles = new Sicore_Models_Server_Db_Files();
            $files = $mFiles->getByFolder($parentUid);
            foreach($files as $file){ 
                if(intval($isDraft)){
                    $draggable = true;
                }else{
                    $draggable = false;
                }
                if($file instanceof Sicore_Models_FSO_File_Image){  
                    $preview = '<img src="'.$file->getThumbnail().'" />';
                }else{
                    $preview = false;
                }
                $data[] = array('attributes' => array('id'         => self::ID_PREFIX_FILE.$file->uid->value(),
                                                      'uid'        => $file->uid->value(), 
                                                      'parent'     => $file->folders_uid->value(), 
                                                      'title'      => '['.$file->uid->value().'] '.($file->title->value() ? $file->title->value() : $file->name->value()),
                                                      'metadata'   => array("draggable" => true),
                                                      'sitype'     => 'file',
                                                      'isDraft'    => intval($isDraft), 
                                                      'metadata'   => array("draggable" => $draggable),
                                                      'object'     => get_class($file),
                                                      'canRename'  => 0,
                                                      'canEdit'    => intval($isDraft),
                                                      'canDelete'  => intval($isDraft),
                                                      'canAddFolder'    => 0,
                                                      'canAddSubFolder' => 0,
                                                      'canUpload'       => 0,
                                                      'preview'         => $preview
                                                    ),         
                              'state'      => 'closed',
                              'data'       => array('title' => $file->name->value(), 
                                                    'icon'  => $file->getIcon()
                                                    ) 
                              );     
            }
            if(empty($parentUid) && $this->config_base->onlinepool->images){
               
                $data[] = array('attributes' => array('id'         => self::ID_PREFIX_FOLDER.'pool_',
                                                      'uid'        => 'pool_', 
                                                      'parent'     => '',
                                                      'folder'     => '', 
                                                      'title'      => $this->view->translate('DESIGN_IMAGES_POOL'),
                                                      'metadata'   => array("draggable" => false),
                                                      'sitype'     => 'folder',
                                                      'isDraft'    => intval($isDraft), 
                                                      'metadata'   => array("draggable" => false),
                                                      'object'     => 'pool',
                                                      'canRename'  => 0,
                                                      'canEdit'    => 0,
                                                      'canDelete'  => 0,
                                                      'canAddFolder'    => 0,
                                                      'canAddSubFolder' => 0,
                                                      'canUpload'       => 0,
                                                      'preview'         => false 
                                                    ),         
                              'state'      => 'closed',
                              'data'       => array('title' => $this->view->translate('DESIGN_IMAGES_POOL'), 
                                                    'icon'  => 'icon_pool.gif'
                                                    ) 
                              );      
            }
        }
        $this->view->data = $data; 
      
    }
    
    public function choosertreeAction()
    {
        $parentUid = $this->getRequest()->getparam('uid');
        $mFolders = new Sicore_Models_Server_Db_Folders();
        $folders = $mFolders->getByParentUid($parentUid);
   
        $data = array();
        
        foreach($folders as $folder){
             
            $data[] = array('attributes' => array('id'         => self::ID_PREFIX_FOLDER.$folder->uid->value(),
                                                  'uid'        => $folder->uid->value(), 
                                                  'parent'     => $folder->uid_parent->value(),
                                                  'title'      => '['.$folder->uid->value().'] '.$folder->name->value(),
                                                  'sitype'     => 'folder'
                                                ),         
                          'state'      => 'closed',
                          'data'       => array('title' => $folder->name->value(), 
                                                'icon'  => 'icon_folder.gif'
                                                ) 
                          );   
        } 
        
        $this->view->data = $data;
    }
    
    public function choosercontentAction()
    {
        $folderUid = $this->getRequest()->getparam('uid');
        if($this->getRequest()->getparam('onlyimages')){
            $class = array('Sicore_Models_FSO_File_Image');
        }else{
            $class = array();  
        }
        $this->view->onlyFolders = (bool)$this->getRequest()->getparam('onlyfolders'); 
                                                                                  
        $this->view->size        = $this->config_base->gfx->sizes->thumb;      
        $mFolders = new Sicore_Models_Server_Db_Folders(); 
        $mFiles   = new Sicore_Models_Server_Db_Files();  
 
        $this->view->folder  = $mFolders->get($folderUid);
        $this->view->files   = $mFiles->getByFolder($folderUid, $class);
        
        $content = $this->view->render('chooser/media.phtml');  
        
        $data    = array('status'  => true,
                         'content' => $content
                        );
                        
        die(Zend_Json::encode($data));
    }  
        
    public function dropimageAction()
    {
        $data = array();
        
        $uid  = intval($this->getRequest()->getParam('uid'));
        $page = intval($this->getRequest()->getParam('page'));
        $lang = $this->getRequest()->getParam('lang');
        
        $mFiles = new Sicore_Models_Server_Db_Files();
        if(!($image  = $mFiles->get($uid))){
            return;
        }
        
        $data['content'] = '<img uid="'.$uid.'" alt=""  name="'.$image->name->value().'" src="'.$image->getPreview().'"  />';
        echo Zend_Json::encode($data);
        die();
    }
    
    public function youtubeAction()
    {        
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);                   
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form   = new Sicore_Models_Gui_Form_Youtube();
        $form->setAction($path.'sicore/media/youtubeinsert');
        $data = array();
        $data['labelInsert']   = $this->view->translate('GENERAL_INSERT');
        $data['labelCancel']   = $this->view->translate('GENERAL_CANCEL');
        $data['title']      = $this->view->translate('GENERAL_INSERT_YOUTUBE');  
        $data['content']    = $form->render();     
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions(); 
        echo Zend_Json::encode($data);
        die();
    }
    
    public function youtubeinsertAction()
    {
        $translate = Sicore_Models_Registry::getTranslate();
        
        $data = array();
        if ($this->_request->isPost()){          
            $url = $this->getRequest()->getparam('url');
            
            if(!preg_match('/(v\=)([a-zA-Z\-_0-9]+)/', $url, $treffer)){
                $data['saved']    = false;
                $data['errors'][] = $translate->_('E_GENERAL_YOUTUBE'); 
                $data['fields'][] = 'url';
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT');    
            }else{
                $data['style']  = $this->getRequest()->getParam('style'); 
                
                $size  = $this->getRequest()->getParam('size'); 
                if(strpos($size, 'x')){
                    $size  = explode('x', $size);
                    $data['width']  = intval($size[0]);
                    $data['height'] = intval($size[1]);
                    $data['style'] = preg_replace('/(width)(\s*:\s*)(\d*)(px)/i', 'width: '.intval($size[0]).'px', $data['style']);
                    $data['style'] = preg_replace('/(height)(\s*:\s*)(\d*)(px)/i', 'height: '.intval($size[1]).'px', $data['style']);   
                }
                
                $data['id']    = $treffer[2]; 
                $data['size']  = $size;  
                $data['saved'] = true;
            }  
        }
        
        echo Zend_Json::encode($data);
        die();
    }
    
    public function iframeAction()
    {        
        $path = Sicore_Models_Helper_FSO::joinPath($this->config_base->website->path);                   
        $path = $this->config_base->website->script ? $path.$this->config_base->website->script.'/' : $path; 
        $form   = new Sicore_Models_Gui_Form_Iframe();
        $form->setAction($path.'sicore/media/iframeinsert');
        $data = array();
        $data['labelInsert']   = $this->view->translate('GENERAL_INSERT');
        $data['labelCancel']   = $this->view->translate('GENERAL_CANCEL');
        $data['title']      = $this->view->translate('GENERAL_INSERT_IFRAME');  
        $data['content']    = $form->render();     
        $data['scripts']    = $this->view->jQuery()->getOnLoadActions();
        echo Zend_Json::encode($data);
        die();
    }
    
    public function iframeinsertAction()
    {
        $translate = Sicore_Models_Registry::getTranslate();
        $data = array();
        if ($this->getRequest()->isPost()){ 
            
            $formData = $this->getRequest()->getPost();
            $form     = new Sicore_Models_Gui_Form_Iframe($formData);  

            if(!$form->isValid($formData)){
                $errors = array();
                if(($subForms = $form->getSubForms())){
                    foreach($subForms as $subForm){
                        $elements =  $subForm->getElements();
                        foreach($elements as $element){
                            if(($error = $element->getDecorator('Errors')->setElement($element)->render(''))){
                                $errors[] = $error;
                                $fields[] = $element->getName();    
                            }
                        }
                    }
                }else{
                    $elements =  $subForms->getElements();
                    foreach($elements as $element){
                        if(($error = $element->getDecorator('Errors')->setElement($element)->render(''))){
                                $errors[] = $error; 
                                $fields[] = $element->getName();    
                            }    
                    }
                }   
                $data['saved']  = false;
                $data['errors'] = $errors;
                $data['fields'] = $fields;
                $data['msg_title'] = $translate->_('GENERAL_FORM_FAILED_TITLE');
                $data['msg_text']  = $translate->_('GENERAL_FORM_FAILED_TEXT');
            } else{
                $data['url']     = str_replace('/', '%2F', $this->getRequest()->getParam('url'));
                $data['width']   = $this->getRequest()->getParam('width'); 
                $data['height']  = $this->getRequest()->getParam('height');
                $data['style']   = $this->getRequest()->getParam('style');
                $data['saved']   = true;
            }   
        }
        echo Zend_Json::encode($data);
        die();
    }
     
}