<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Gui.php 1207 2010-02-27 19:03:54Z ms $
 */
class Sicore_Models_Gui
{
    /**
     * Template zum Rendern des Objektes
     *
     * @var String
     */
    protected $_template = null;
    
    /**
     * Javascript-Datei
     *
     * @var String
     */
    protected $_jsFile = null;
    
    /**
     * Stylesheet-Datei
     *
     * @var String
     */
    protected $_cssFile = null;
    
    /**
     * Alternative Stylesheet-Datei
     *
     * @var String
     */
    protected $_cssAlternateFiles = array();
    
    /**
     * View-Objekt
     *
     * @var Zend_View
     */
    protected static $_view = null;
    
    /**
     * Allgemeine Konfiguration
     *
     * @var Zend_Config_Ini
     */
    protected static $_config_base = null;
    
    /**
     * Backend-Konfiguration
     *
     * @var Zend_Config_Ini
     */
    protected static $_config_backend = null;
    
    /**
     * Konstruktor
     *
     */
    public function __construct($template = null, $jsFile = null, $cssFile = null)
    {
        $this->_template  = $template; 
        $this->_jsFile    = $jsFile;
        $this->_cssFile   = $cssFile;
                
        $this->init();
    }
    
    /**
     * Starten von weiteren Prozessen in den Unterklassen
     *
     * @return Sicore_Models_Gui
     */
    public function init()
    {
        return $this;
    }
    
    /**
     * Setzt die View als statische Klassenvariable
     *
     * @param Zend_View $view
     */
    public static function setView(Zend_View $view)
    {
        self::$_view = $view;
    }
    
    /**
     * Setzt die allgemeine Konfiguration als statische Klassenvariable
     *
     * @param Zend_Config_Ini $_config_base
     */
    public static function setConfigBase(Zend_Config $_config_base)
    {
        self::$_config_base = $_config_base;
    }
    
    /**
     * Setzt die Backend-Konfiguration als statische Klassenvariable
     *
     * @param Zend_Config_Ini $_config_backend
     */
    public static function setConfigBackend(Zend_Config $_config_backend)
    {
        self::$_config_backend = $_config_backend;
    }
    
    /**
     * Fügt Javascript- und Stylesheet-Dateien der View hinzu
     *
     */
    public function appendFilesToView()
    {
        $path = Sicore_Models_Helper_FSO::joinPath(self::$_config_base->website->path); 
        //JS-Datei hinzufügen
        if(self::$_view->headScript() && $this->_jsFile){
            $file = $path.'application/Sicore/Javascript/'.$this->_jsFile;
            self::$_view->headScript()->appendFile($file);
        }
        
        //CSS-Dateien aus dem obersten Design-Verzeichnis hinzufügen
        if(self::$_view->headLink() && $this->_cssFile){
            $file = $path.'templates/backend/'.self::$_config_backend->layout.'/_css/'.$this->_cssFile;
            self::$_view->headLink()->appendStylesheet($file);
        }
        
        //Alernative CSS-Dateien aus dem obersten Design-Verzeichnis hinzufügen
        if(self::$_view->headLink() && count($this->_cssAlternateFiles) > 0){
            foreach($this->_cssAlternateFiles as $css){
                $file = $path.'templates/backend/'.self::$_config_backend->layout.'/_css/'.$css->file;
                self::$_view->headLink()->appendStylesheet($file, $css->media, $css->conditionalComment);
            } 
        }
    }
    
    public function addAlternateStylesheet($file, $conditionalComment, $media = 'screen'){
        $css = new stdClass();
        $css->file = $file;
        $css->conditionalComment = $conditionalComment;
        $css->media = $media; 
        $this->_cssAlternateFiles[] = $css;
    }

    
    /**
     * Fügt Javascript- und Stylesheet-Dateien der View hinzu
     *
     */
    public function render()
    {
        if(!$this->_template){
            return;
        }
        self::$_view->template = $this->_template;
        $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer'); 
        $viewRenderer->setScriptAction($this->_template); 
        $viewRenderer->setNoController(true); 
    }
    
    /**
     * Fügt eine Variable dem Template (View-Objekt) hinzu
     *
     * @param String $key
     * @param String $value 
     * @return Sicore_Models_Gui
     */
    public function append($key, $value)
    {
        self::$_view->$key = $value;
    }
    
    
}

