<?php
  /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Backend.php 1557 2010-10-21 16:50:11Z ms $
 */
class Sicore_Models_Gui_Form_Backend extends Sicore_Models_Gui_Form
{
    protected $_node = null;
    protected $_user = null;
    const IDPREFIX = 'siform_'; 
    
    public function __construct(Sicore_Models_Database $node, Sicore_Models_Uobj $user, $doNotFilter = false, $isCreateMode = false)
    {
        $this->_node = $node;
        $this->_user = $user;
        
        parent::__construct($doNotFilter, $isCreateMode);
    }
        
    public function init()
    {
        $this->setAttrib('id',     'si_edit_main'); 
        $this->addElementPrefixPath('Simploo_Filter', 'Simploo/Filter', 'filter');
        
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        if(($subforms = $this->_node->getSubforms())){
            $this->setDecorators(array(
                'FormElements',
                array('TabContainer', array(
                    'id'          => 'tabContainer'
                )),
                'Form',
            ));
            foreach($subforms as $key => $subform){
                $fields    = $subform['fields'];
                 
                $subForm = new ZendX_JQuery_Form(array('class' => 'EMIL'));
   
                $subForm->clearDecorators();
                $subForm->setDecorators(array(
                    'FormElements',
                    array('TabPane',
                          array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                        'title' => strtoupper(get_class($this->_node)."_".$key),
                                                        ),
                                                        
                          )
                    )      
                                                        
                )); 
                
                foreach($fields as $field){
                    if(isset($this->_node->$field)) continue;  
                    $property = $this->_node->$field;
                    if(!($property instanceof Sicore_Models_Node_Property)) continue;
                    $config   = $property->getFormConfig();
                    
                    $element = new $config->type($property->field());
                    //$element2 = new Zend_Form_Element_Text('test');
                    //$element->addFilter(new Zend_Filter_StripTags());
                    //$element->addFilter(new Zend_Filter_HtmlEntities(ENT_QUOTES, 'utf-8'));
                    if($config->type == 'Simploo_JQuery_Form_Element_Drop'){
                        $element->setRegisterInArrayValidator(false);
                    }
                    
                    if($this->_isCreateMode && $config->create instanceof Zend_Config){
                        $config->merge($config->create);  
                    }
                    if($this->_node instanceof Sicore_Models_Uobj_User && !$this->_node->level->value() && $config->rootadmin instanceof Zend_Config){
                        $config->merge($config->rootadmin); 
                    }
                       
                    $element->setConfig($config);
                    
                    if($config->validators){
                        foreach($config->validators as $validator){
                            if(($validator->validator == 'Db_NoRecordExists' || $validator->validator == 'Db_RecordExists') && $validator->excludeSelf){
                                $name = 'Zend_Validate_'.$validator->validator;
                                $element->removeValidator($validator->validator); 
                                $element->addValidator(new $name($validator->options->table, $validator->options->field, 'uid != '.intval($this->_node->uid->value())));
                           }
                        }
                    }
                    if($config->readonly){
                        $element->setAttrib('readonly', 'readonly');
                    }else{
                        $element->setAttrib('readonly', null); 
                    }   
                    
                    if($this->doNotFilter()){
                        $element->clearFilters() ;
                    }
                    
                    if($config->type == 'Simploo_JQuery_Form_Element_MultiInput'){
                        $element->setValue($this->_node->$field->getRawData());
                        if(isset($config->options)){
                            $element->addMultiOptions($config->options->toArray());
                        }
                    }elseif($config->type == 'Simploo_JQuery_Form_Element_MultiSelect'){
                        $element->setValue($this->_node->$field->toArray());
                        if(isset($config->options)){
                            $element->addMultiOptions($config->options->toArray());
                        }   
                    }elseif(!($property instanceof Sicore_Models_Node_Property_Password)){ 
                        $element->setValue($this->_node->$field->value()); 
                    }
                    if($property instanceof Sicore_Models_Node_Property_Join_Interface){
                        $element->addMultiOptions($property->getJoinedData());  
                    }
                    $cssClass = $config->class ? get_class($property).' '.$config->class : get_class($property);
                    $element->setAttrib('class', $cssClass);
                    if($config->type != 'Zend_Form_Element_Hidden'){
                        if($config->label){
                            $element->setLabel($config->label);
                        }else{
                            $element->setLabel(strtoupper(get_class($this->_node)."_".$property->field()));
                        }
                    }
                 
                    if($config->type == 'ZendX_JQuery_Form_Element_DatePicker' || $config->type == 'Simploo_JQuery_Form_Element_DateTimePicker'){
                        $months_keys = array(0,1,2,3,4,5,6,7,8,9,10,11);
                        $months      = $locale->getTranslationList('months');
                        $days        = $locale->getTranslationList('days');
                        $days_min = array(0 => substr($days['format']['abbreviated']['sun'], 0, 2),
                                          1 => substr($days['format']['abbreviated']['mon'], 0, 2),
                                          2 => substr($days['format']['abbreviated']['tue'], 0, 2),
                                          3 => substr($days['format']['abbreviated']['wed'], 0, 2),
                                          4 => substr($days['format']['abbreviated']['thu'], 0, 2),
                                          5 => substr($days['format']['abbreviated']['fri'], 0, 2),
                                          6 => substr($days['format']['abbreviated']['sat'], 0, 2)
                                          );
                       $days_wide = array(0 => $days['format']['wide']['sun'],
                                          1 => $days['format']['wide']['mon'],
                                          2 => $days['format']['wide']['tue'],
                                          3 => $days['format']['wide']['wed'],
                                          4 => $days['format']['wide']['thu'],
                                          5 => $days['format']['wide']['fri'],
                                          6 => $days['format']['wide']['sat']);
                        $element->setJQueryParam('dateFormat', 'dd.mm.yy');  
                        $element->setJQueryParam('monthNames', array_combine($months_keys, $months['format']['wide']));       
                        $element->setJQueryParam('monthNamesShort', array_combine($months_keys, $months['format']['abbreviated']));
                        $days   = $locale->getTranslationList('days'); 
                        $element->setJQueryParam('dayNames', $days_wide);       
                        $element->setJQueryParam('dayNamesShort', $days_min);
                        $element->setJQueryParam('dayNamesMin', $days_min );
                        $element->setJQueryParam('closeText', $translate->translate('GENERAL_CANCEL') );     
                        $element->setJQueryParam('currentText',   $translate->translate('GENERAL_TODAY'));
                        $element->setJQueryParam('showButtonPanel',  true);                          
                    } 
                    if(get_class($property) == 'Sicore_Models_Node_Property_Datetime'){
                        $locale = Sicore_Models_Registry::getLocale();
                        $dateValidator = new Zend_Validate_Date(Zend_Date::DATE_MEDIUM, $locale);
                        $element->addValidator($dateValidator);
                    }  
                    $element->removeDecorator('HtmlTag');
                    if($config->required && $config->disable){
                        $element->setRequired(false);
                        $element->addDecorator('Label', array('class' => 'required')); 
                    }else{
                        $element->addDecorator('Label'); 
                    }                 
                    $element->addDecorator('Description'); 
                    $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));   
                    $element->setAttrib('id', self::IDPREFIX.$property->field()); 
                    if($subform['admin'] && !$this->_user->isadmin->value()){
                        $element->setAttrib('disabled', 'disabled');
                    }
                    $subForm->addElement($element, $property->field());
                    
                    //$subForm->addElement('hash', 'si_hash_'.$key, array('salt' => 'unique'));
                }
                $this->addSubForm($subForm, $key);  
            }                     
        }else{
            foreach($this->_node as $property){
                $config  = $property->getFormConfig();
                $element = new $config->type($property->field());
                $element->setConfig($config);
                $elements[] = $element;
            }

            $this->addElements($elements);
        } 
    }   
}