<?php
 /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Image.php 1427 2010-06-15 14:13:16Z ms $
 */
class Sicore_Models_Gui_Form_Image extends Sicore_Models_Gui_Form
{
    
    protected $_file  = null;
    protected $_name  = null;
    protected $_alt   = null;  
    protected $_title = null;
    protected $_uid   = null; 
    protected $_image = null;
    protected $_superbox = false;
    
    const IDPREFIX = 'siform_';  
    
    public function __construct($uid, $image, $file = '', $name = '', $alt = '', $title = '', $superbox = false)
    {        
        $this->_uid   = $uid;
        $this->_file  = $file; 
        $this->_name  = $name; 
        $this->_alt   = $alt; 
        $this->_title = $title; 
        $this->_image = $image;
        $this->_superbox = $superbox;
        
        parent::__construct();
    }
    
    public function init()
    {
        $this->setAttrib('id',     'si_edit_main'); 
        
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('GENERAL_INSERTIMAGE'))))
        ));
        
        $config_base     = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_BASE); 
        if($config_base->gfx->content){
            $imagesizes = explode(',', $config_base->gfx->content);   
        }else{
            $imagesizes = array('preview');
        }
        
        $element = new Zend_Form_Element_Select('imagesize');
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_IMAGESIZE');
        foreach($imagesizes as $imagesize){
            $element->addMultiOption($imagesize, 'GENERAL_GFX_IMAGES_'.strtoupper($imagesize));   
        }  
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'imagesize');  
        $subForm->addElement($element, 'imagesize'); 
        
        $element = new Zend_Form_Element_Checkbox('usesuperbox');
        $element->setLabel('GENERAL_HTML_IMAGECLICK');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'usesuperbox');  
        $subForm->addElement($element, 'usesuperbox');
        
         $element = new Zend_Form_Element_Text('alt');
        $element->setValue($this->_alt); 
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_HTML_ALT');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'alt');  
        $subForm->addElement($element, 'alt');  
        
        $element = new Zend_Form_Element_Text('title');
        $element->setValue($this->_title); 
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_HTML_TITLE');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'title');  
        $subForm->addElement($element, 'title');
        
        $element = new Zend_Form_Element_Image('image');
        $element->setLabel('GENERAL_HTML_IMAGE'); 
        $element->setImage($this->_image);
        $element->setAttrib('id', self::IDPREFIX.'image');  
        $subForm->addElement($element, 'image');
                
        $element = new Zend_Form_Element_Text('file');
        $element->setValue($this->_file);
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('SICORE_MODELS_FSO_FILE_NAME');
        $element->disabled = true;
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'filename');  
        $subForm->addElement($element, 'filename');  
        
        $element = new Zend_Form_Element_Hidden('style');
        $element->setAttrib('id', self::IDPREFIX.'style');  
        $subForm->addElement($element, 'style'); 
        
        $element = new Zend_Form_Element_Hidden('width');
        $element->setAttrib('id', self::IDPREFIX.'width');  
        $subForm->addElement($element, 'width'); 
        
        $element = new Zend_Form_Element_Hidden('height');
        $element->setAttrib('id', self::IDPREFIX.'height');  
        $subForm->addElement($element, 'height'); 
       
        $element = new Zend_Form_Element_Hidden('oldimagesize');
        $element->setValue($this->_uid);      
        $element->setAttrib('id', self::IDPREFIX.'oldimagesize');  
        $subForm->addElement($element, 'oldimagesize'); 
        
        $element = new Zend_Form_Element_Hidden('uid');
        $element->setValue($this->_uid);      
        $element->setAttrib('id', self::IDPREFIX.'uid');  
        $subForm->addElement($element, 'uid');  
        
        $this->addSubForm($subForm, 'insert');  

    }   
}