<?php
      /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Gui
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Link.php 1307 2010-04-27 13:20:10Z ms $
 */
class Sicore_Models_Gui_Form_Link extends Sicore_Models_Gui_Form
{
    
    protected $_uid    = null;
    protected $_target = null;
    protected $_title  = null;
    protected $_name   = null; 
    protected $_type   = null;
    
    const PAGE  = 'page';
    const IMAGE = 'image';  
    const FILE  = 'file';
    const EXTERNAL  = 'extern';
    const EMAIL  = 'email';
    
    const IDPREFIX = 'siform_'; 
    
    public function __construct($type, $uid = null, $name = null, $title = null, $target = null)
    {        
        $this->_uid    = $uid;
        $this->_name   = $name;  
        $this->_title  = $title;
        $this->_target = $target; 
        $this->_type   = $type; 
        parent::__construct();
    }
    
    public function init()
    {
        
        $this->setAttrib('id',     'si_edit_main'); 
        $locale    = Sicore_Models_Registry::getLocale();
        $translate = Sicore_Models_Registry::getTranslate(); 
        
        $this->setDecorators(array(
            'FormElements',
            array('TabContainer', array(
                'id'          => 'tabContainer'
            )),
            'Form',
        ));
        
        //INTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('GENERAL_INSERTLINK_INTERN'))))
        ));
               
        $element = new Zend_Form_Element_Text('name');
        $element->setValue($this->_name);
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_LINKTARGET');
        $element->setDescription($translate->_('N_GENERAL_LINKTARGET')); 
        $element->disabled = true;
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'name');  
        $subForm->addElement($element, 'name'); 
        
        $element = new Zend_Form_Element_Text('title');
        $element->setValue($this->_title); 
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_HTML_TITLE');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'title');  
        $subForm->addElement($element, 'title'); 
       
        $element = new Zend_Form_Element_Select('target');
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_OPNEIN');
        $element->addMultiOption('_self', 'GENERAL_LINKTARGET_SELF');
        $element->addMultiOption('_blank', 'GENERAL_LINKTARGET_NEW');
        switch($this->_type){
            case self::IMAGE:
                $element->addMultiOption('superbox[gallery][page]', 'GENERAL_LINKTARGET_SUPER');
                break; 
            case self::PAGE: 
                $element->addMultiOption('superbox[iframe][600x400]', 'GENERAL_LINKTARGET_SUPER');
                break; 
            default:
                break;    
        } 
         
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'target');  
        $subForm->addElement($element, 'target'); 
         
        $this->addSubForm($subForm, 'insert_intern');  

        //EXTERN
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('GENERAL_INSERTLINK_EXTERN'))))
        ));
        
        $element = new Zend_Form_Element_Text('extern_url');
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_LINKURL');
        $element->setDescription($translate->_('N_GENERAL_LINKURL'));  
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'extern_url');  
        $subForm->addElement($element, 'extern_url'); 
        
        $element = new Zend_Form_Element_Text('extern_title');
        $element->setValue($this->_title); 
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_HTML_TITLE');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'extern_title');  
        $subForm->addElement($element, 'extern_title'); 
       
        $element = new Zend_Form_Element_Select('extern_target');
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_OPNEIN');
        $element->addMultiOption('_self', 'GENERAL_LINKTARGET_SELF');
        $element->addMultiOption('_blank', 'GENERAL_LINKTARGET_NEW');
        $element->addMultiOption('superbox[iframe][600x400]', 'GENERAL_LINKTARGET_SUPER');

         
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'extern_target');  
        $subForm->addElement($element, 'extern_target'); 
        
        $element = new Zend_Form_Element_Hidden('extern');
        $element->setValue(0); 
        $element->setAttrib('id', self::IDPREFIX.'extern');  
        $subForm->addElement($element, 'extern');
         
        $this->addSubForm($subForm, 'insert_extern'); 
        
        //E-MAIL
        $subForm = new ZendX_JQuery_Form();
        $subForm->clearDecorators();
        $subForm->setDecorators(array(
            'FormElements',
            array('TabPane',
                  array('jQueryParams' => array('containerId' => 'si_edit_main',
                                                'title' => $translate->_('GENERAL_INSERTLINK_EMAIL'))))
        ));
        
        $element = new Zend_Form_Element_Text('email_url');
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_EMAIL');   
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'email_url');  
        $subForm->addElement($element, 'email_url'); 
        
        $element = new Zend_Form_Element_Text('email_title');
        $element->setValue($this->_title); 
        $element->addFilter(new Simploo_Filter_StripSlashes()); 
        $element->setLabel('GENERAL_HTML_TITLE');
        $element->removeDecorator('HtmlTag');
        $element->addDecorator('Label');  
        $element->addDecorator('Description'); 
        $element->addDecorator('HtmlTag', array('tag' => 'div', 'class' => 'si_element', 'suffix' => ':'));
        $element->setAttrib('id', self::IDPREFIX.'email_title');  
        $subForm->addElement($element, 'email_title'); 
        
        $element = new Zend_Form_Element_Hidden('email');
        $element->setValue(0); 
        $element->setAttrib('id', self::IDPREFIX.'email');  
        $subForm->addElement($element, 'email');
        
        $element = new Zend_Form_Element_Hidden('uid');
        $element->setValue($this->_uid); 
        $element->setAttrib('id', self::IDPREFIX.'uid');  
        $subForm->addElement($element, 'uid'); 
               
        $this->addSubForm($subForm, 'insert_email'); 
        
    }   
}