<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Helper
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Link.php 1538 2010-09-11 08:33:56Z ms $
 */

class Sicore_Models_Helper_Link
{
    const WEB_PATH = 1;
    const ABS_PATH = 2;
    /**
     * Basis-URL für die Links, z.B. /testdir
     *
     * @var String
     */
    protected $_baseUrl   = array();
    
    /**
     * Zusätzliche URL-Parameter
     *
     * @var Array
     */
    public $_params   = array();
    
    /**
     * Sprachversion für den Link
     *
     * @var String
     */
    protected $_language = null;
    
    /**
     * Entwurfsansichz
     *
     * @var Boolean
     */
    protected $_isDraft  = false;
    
    /**
     * Konstante zum Generieren der Parameter im Zend-Format
     *
     * @var Integer
     */
    const ZEND    = 1;
    
    /**
     * Konstante zum Generieren der Parameter im Simploo-Format
     *
     * @var Integer
     */
    const SIMPLOO = 2;
    
    /**
     * Konstruktor
     *
     * @param Zend_Config $config Konfiguration aus base.ini.php
     */ 
    public function __construct($baseUrl = '')
    {
        if(substr($baseUrl, 0, 1) != '/'){
            $baseUrl = '/'.$baseUrl;
        }
        if(substr($baseUrl, strlen($baseUrl)-1) == '/'){
            $baseUrl = substr($baseUrl, 0, strlen($baseUrl)-1);
        }
        $this->_baseUrl = $baseUrl; 
    }
    
    /**
      * Gibt einen externen Link zurück
      *
      * @param String link
      * @return String
      */
   public static function getExternalLink($url)
   {
       $url = strip_tags($url); 
       $url = str_replace('"', '', $url);
       $url = stripslashes($url);  
       
       if(empty($url)){
           return false;           
       }
       
       if(preg_match('/^.+:\/\//i', $url)){
            return $url;
        }else{
            return "http://$url";
        }
   }
   
    /**
      * Gibt einen E-Mail-Link zurück
      *
      * @param String link
      * @return String
      */
   public static function getEmailLink($email)
   {
       $email = strip_tags($email); 
       $email = str_replace('"', '', $email);
       $email = stripslashes($email);  
       
       if(empty($email)){
           return false;           
       }
       
       if(!preg_match('/^[^@]+@{1}[^@]+\.\w+$/i', $email)){
           return false;
       }
       
       if(preg_match('/^mailto:/i', $email)){
            return $email;
       }else{
           return "mailto:$email";
       }
   }
    
    /**
      * Fügt einen URL-Parameter hinzu
      *
      * @param String $key
      * @param Mixed $value
      * @return $this
      */
    public function addParam($key, $value)
    {
        if(!empty($value)){
            $this->_params[(string)$key] = $value;
        }   
        return $this;
    }
   
    /**
      * Setzt die URL-Parameter
      *
      * @param Array $params 
      * @return $this
      */
    public function setParams(array $params)
    {     
        $this->_params = $params;
        return $this;
    } 
   
    /**
      * Setzt die Sprachversion
      *
      * @param String $lang 
      * @return $this
      */
    public function setLanguage($lang)
    {
        $this->_language = (string)$lang;
        
        return $this;
    }
   
    /**
      * Setzt die Entwurfsversion
      *
      * @param Boolean $isDraft 
      * @return $this
      */
    public function setDraft($isDraft){
        $this->_isDraft = $isDraft; 
        $this->addParam('draft', intval($this->_isDraft));
        return $this;
    }
    

   
    /**
      * Gibt die Linkdadresse anhand eines Knotens aus
      *
      * @param Sicore_Models_Node $node 
      * @return String
      */
    public function getByNode(Sicore_Models_Node $node)
    {
    
        if(($node instanceof Sicore_Models_Node_Page) && ($external = $node->external->value())){
            return $external;
        }
        
        $url = $node->permalink->value() ? $node->permalink->value() : $node->getUrlalias()->alias->value();
      
        $configBase = Sicore_Models_Registry::getConfig();
        if($configBase->website->script){
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $configBase->website->script); 
        }else{
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path); 
        }
        if(empty($url)){
            return $node->getTree()->level->value() > 0 
                   ? $this->getByUid($node->uid->value()) 
                   : $path.substr($this->_getParameterString(self::SIMPLOO), 1);
        }else{
            return $path.$url.$this->_getParameterString(self::SIMPLOO); 
        }
    }
   
    /**
      * Gibt die Linkdadresse anhand eines Knoten-Alias aus
      *
      * @param String $alias 
      * @return String
      */
    public function getByAlias($alias)
    {
        $configBase = Sicore_Models_Registry::getConfig();
        if($path.$configBase->website->script){
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $path.$configBase->website->script); 
        }else{
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path); 
        }
        return $path.$alias.$this->_getParameterString(self::SIMPLOO);  
    }
   
    /**
      * Gibt die Linkdadresse anhand einer Knoten-ID aus
      *
      * @param Integer $uid 
      * @return String
      */
    public function getByUid($uid)
    {
        $uid = intval($uid);
        $configBase = Sicore_Models_Registry::getConfig();

        if($configBase->website->script){
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $configBase->website->script); 
        }else{
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path); 
        } 
        return $path."sifront/page/show/uid/$uid".$this->_getParameterString(); 
    }
    
    /**
      * Gibt die Linkdadresse anhand einer Knoten-ID aus
      *
      * @param Integer $uid 
      * @return String
      */
    public function getByFileUid($uid, $mode = null, $realPath = 0)
    {
        $uid = intval($uid);
        $configBase     = Sicore_Models_Registry::getConfig();
        $configFrontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND);
        
        $modelFile = new Sicore_Models_Server_Db_Files();
        $file      = $modelFile->get($uid);
        if(!$file) return false;
            
        //Bild überprüfen und gegebenfalls erstellen
        $path = Sicore_Models_Helper_FSO::joinPathAbs('data/files', $file->folders_uid->value(), $file->uid->value());
        $name = $mode ? $mode.'_'.$file->name->value() : $file->name->value();   
        if($file instanceof Sicore_Models_FSO_File_Image && !file_exists($path.$name)){
            if(!($size = $configBase->gfx->sizes->$mode)){
                return false;
            }
            $mImage = new Sicore_Models_Image(); 
            $mImage->convertFromConfig($path.$file->name->value(), $path.$name, $size);
        }
        if(!($file instanceof Sicore_Models_FSO_File_Image)){
            $name = $file->name->value();
            $mode = null;
        }
        
        //Ausgabe Dateilink
        $realPath       = !$realPath && $configFrontend->download->method == 'direct' ? self::WEB_PATH : $realPath; 
        if(!$realPath){
            $mode = $mode ? "/[mode]$mode" : ''; 
            if($configBase->website->script){
                return Sicore_Models_Helper_FSO::joinPath($path, $configBase->website->script, 'sifront/file/download')."[id]".$uid.$mode; 
            }else{
                return Sicore_Models_Helper_FSO::joinPath($path, 'sifront/file/download')."[id]".$uid.$mode; 
            }
        }else{ 
            $name = $mode ? $mode.'_'.$file->name->value() : $file->name->value(); 
            if($realPath == self::WEB_PATH ){
                $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, 'data/files', $file->folders_uid->value(), $file->uid->value());
            }elseif($realPath == self::ABS_PATH ){
                $path = Sicore_Models_Helper_FSO::joinPathAbs('data/files', $file->folders_uid->value(), $file->uid->value());
            }else{
                return null;
            }
            //$path = $configBase->website->script ? '/'.$configBase->website->script.$path : $path;  
            return $path.$name; 
        }
    }
    
   
    /**
      * Gibt die Linkdadresse des Root-Knotens aus
      *
      * @return String
      */
    public function getHome()
    {
          
        $configBase = Sicore_Models_Registry::getConfig();
        if($configBase->website->script){
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $configBase->website->script);
        }else{
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path);
        }        
        return $path.substr($this->_getParameterString(self::SIMPLOO), 1);
    }
   
    /**
      * Gibt eine manuelle Zend-Formatierte Linkadresse aus
      *
      * @param String $module z.B. sicore
      * @param String $controller optional
      * @param String $action optional
      * @return String
      */
    public function get($module, $controller = '', $action = '')
    {
        $configBase = Sicore_Models_Registry::getConfig();
        if($configBase->website->script){
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $configBase->website->script, $module, $controller, $action);
        }else{
            $path = Sicore_Models_Helper_FSO::joinPath($configBase->website->path, $module, $controller, $action);
        }
        $path = substr($path, 0, strlen($path) - 1);
        return $path.$this->_getParameterString();
    }
    
    private function _getParameterString($format = self::ZEND)
    {
        $paramString = '';        
        foreach($this->_params as $key=>$value){
            if($format == self::ZEND){
                $paramString .= "/$key/$value";
            }elseif($format == self::SIMPLOO){
                $paramString .= '/['.$key.']'.$value;
            }
        }
        
        return $paramString;
    }

}
?>
