<?php
      /**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Helper
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Uri.php 1502 2010-08-02 12:54:10Z ms $
 */
class Sicore_Models_Helper_Uri
{
    
    public $isZendFormatted      = false;
    private $_config             = array();
    private $_rawUri             = '';
    private $_rawCoreString      = '';
    private $_rawParamString     = '';
    private $_zendCoreUri        = '';
    private $_zendParamUri       = '';
    private $_params             = array();  
     
    public function __construct( Zend_Config $config, $uri = '')
    {
        $this->_config = $config;
        $this->set($uri);
    }
    
    public function set($uri)
    {
        if($this->_config->website->script){
            $uri = substr($uri, strlen($this->_config->website->script) + 1); 
        } 
        $uri = substr($uri, strlen($uri)-1) == '/' ? substr($uri, 0, strlen($uri)-1) : $uri;
                
        if(substr($uri, 0, 1) == '_'){
            $this->isZendFormatted = true; 
        } else{
            foreach($this->_config->general->modules as $module){
                if(strtolower(substr($uri, 0, strlen($module))) == strtolower($module)){
                    $this->isZendFormatted = true;
                    $uri = ucwords($uri);
                    break;
                }
            }
        }
        
        $this->_rawUri         = $uri;
        $this->_rawCoreString  = $uri;
        $this->_rawParamString = '';
        $this->_zendCoreUri    = $uri;
        $this->_zendParamUri   = '';
        
        $this->_rewrite();
    }
    
    public function rewriteUrlAlias(Sicore_Models_Server_Db_Urlalias_Interface $mUralias, $dModule = 'Sifront', $dController = 'page', $dAction = 'show')
    {      
        
        $isDraft = isset($this->_params['draft']) ? (bool)$this->_params['draft'] : false;
        
        $node = $mUralias->getNode($this->_rawCoreString, $isDraft);
        $this->_zendCoreUri  = "$dModule/$dController/$dAction";   
        if(!$node) return;
        
        $uri = '/uid/'.$node->uid;
        
        if(!array_key_exists('draft', $this->_params)){
            $uri .= '/draft/'.intval($node->version > 0);  
        }
        if(!array_key_exists('lang', $this->_params)){
            $uri .= '/lang/'.$node->language_code; 
        } 
        $this->_zendParamUri = $uri.$this->_zendParamUri; 
    }
    
    
    public function getZendUri()
    {
        return $this->_zendCoreUri.$this->_zendParamUri;
    }
   
    private function _rewrite()
    {
        $this->_rawUri = substr($this->_rawUri, 0, 1) == '[' ? '/'.$this->_rawUri : $this->_rawUri; 
        if(strpos($this->_rawUri, '?')){
            $this->_rawCoreString = substr($this->_rawUri, 0, (strpos($this->_rawUri, '?')));    
        }
        if(strpos($this->_rawUri, '/?')){
            $this->_rawCoreString = substr($this->_rawUri, 0, (strpos($this->_rawUri, '/?')));    
        } 
        
        if(empty($this->_rawUri) || strpos($this->_rawUri, '[') === false) return;
            
        $this->_rawCoreString  = substr($this->_rawUri, 0, (strpos($this->_rawUri, '/[')));
        
        $this->_rawParamString = substr($this->_rawUri, (strpos($this->_rawUri, '/[')));
        $parts = explode('/[', $this->_rawParamString);
        $this->_zendCoreUri = $this->_rawCoreString;
        foreach($parts as $rawParam){
            if(!($param = $this->_explodeParam($rawParam))) continue;
            if($param->key == 'uid') continue;
            $this->_zendParamUri .= "/$param->key/$param->value";
            $this->_params[$param->key] = $param->value;
        }
               
    }
    
    private function _explodeParam($rawParam)
    {
        $parts = explode(']', $rawParam);
        $param = new stdClass();
        if(count($parts) != 2) return null;
        
        $param->key   = $parts[0];
        if(strpos($parts[1], '/') !== false){
            $parts[1] = substr($parts[1], 0, strpos($parts[1], '/')); 
        }
        $param->value = $parts[1];
        
        return $param; 
    }
}
?>
