<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Node
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Property.php 1563 2010-10-27 13:11:07Z ms $
 */
abstract class Sicore_Models_Node_Property{
    
    /**
     * Verweis zum Knoten
     *
     * @var Sicore_Models_Node
     */
    protected $_node          = null;
    
    /**
     * UID des Knoten
     *
     * @var Integer
     */
    protected $_nodeUid        = null;
    
    /**
     * UID des Ursprungsknoten bei vererbten Eigenhschaften
     *
     * @var Integer
     */
    protected $_rootUid       = null;
    /**
     * Property wurde geändert
     *
     * @var Integer
     */
    protected $_canPublish    = false;
    /**
     * Subfields sollen gelöscht werden
     *
     * @var Integer
     */
    protected $_deleteSubfields    = false;
    /**
     * Interner Name in der Datenbank
     *
     * @var String
     */
    protected $_dbFieldName    = null;
    /**
     * Inhalt des Datenbankfeldes
     *
     * @var String
     */
    protected $_value         = null;
    /**
     * Alias des Feldes
     *
     * @var String
     */
    protected $_alias         = null;
    /**
     * Konigurations des entsprechenden Formularfeldes
     *
     * @var Zend_Config
     */
    protected $_formConfig    = array();
    /**
     * Konigurations
     *
     * @var Zend_Config
     */
    protected $_config    = array();
    /**
     * Feld wird nicht in der Tabelle gespeichert, nur Anzeige
     *
     * @var Boolean
     */
     protected $_doNotSave    = false;    
    /**
     * Konstruktor
     *
     * @param TYPE $name
     * @return TYPE
     */
     
     public function __construct($dbFieldName, Sicore_Models_Database $node, $alias)
     {
         $this->_dbFieldName     = $dbFieldName;
         $this->_alias           = $alias;   
         //$this->_node            = $node;
     }

     public function setNodeUid($uid)
     {
         $this->_nodeUid = intval($uid);
     }
     
     public function isInRootline()
     {
         return isset($this->_config->rootline) && $this->_config->rootline;
     }
     
     public function isGlobal()
     {
         return isset($this->_config->global) && $this->_config->global;
     }
     
     public function getRootUid()
     {
         return $this->_rootUid;
     }
          
     public function value($cleanHtml = false)
     {
         if($cleanHtml){
             $config_frontend = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
             if(strtoupper(substr($config_frontend->doctype, 0, 5)) == 'XHTML' && strpos(strtoupper($config_frontend->doctype), 'STRICT') !== false){
                 $this->_value = preg_replace('/(\w+=)([^\s|>|\/|"]+)(\s|>|\/+)/i', '$1"$2"$3', $this->_value);
                 $this->_value = preg_replace('/target="[^"]*"/i','', $this->_value); 
                 $this->_value = preg_replace('/_superboxgroupkey="[^"]*"/i','', $this->_value);
                 $this->_value = preg_replace('/_relsettings="[^"]*"/i','', $this->_value);
             }elseif(strtoupper(substr($config_frontend->doctype, 0, 5)) == 'XHTML'){
                $this->_value = preg_replace('/(\w+=)([^\s|>|\/|"]+)(\s|>|\/+)/i', '$1"$2"$3', $this->_value);
                $this->_value = str_replace('target="superbox[gallery][page]"', '', $this->_value); 
                $this->_value = preg_replace('/_superboxgroupkey="[^"]*"/i','', $this->_value);
                $this->_value = preg_replace('/_relsettings="[^"]*"/i','', $this->_value);
             }
             $this->_value = preg_replace('/<([^>]*)(style="")/i','<$1', $this->_value);     
             if(function_exists("tidy_parse_string") && $config_frontend->usetidy){
                 $config = array('uppercase-attributes' => false,
                                'indent' => true,
                                'drop-proprietary-attributes' => true,
                                'alt-text' => '',
                                'show-body-only' => true,
                                'lower-literals' => true,
                                'fix-uri' => true
                                
                               );
                if(strtoupper(substr($config_frontend->doctype, 0, 5)) == 'XHTML'){
                    $config['output-xhtml'] = true;                   
                }
                $tidy = tidy_parse_string($this->_value, $config, $config_frontend->charset);
                $tidy->cleanRepair();
                $this->_value = $tidy;
                
            }else{   
                $config_base = Sicore_Models_Registry::getConfig(); 
                $path = Sicore_Models_Helper_FSO::joinPath($config_base->website->path);
                $this->_value = preg_replace('/<br[^>]*>/i','<br/>', $this->_value);
                $this->_value = preg_replace('/<img([^>]*)>/i','<img$1/>', $this->_value); 
                $this->_value = preg_replace('/uid="[^"]*"/i','', $this->_value);
                $this->_value = preg_replace('/name="[^"]*"/i','', $this->_value);
                $this->_value = preg_replace('/imagesize="[^"]*"\s*/i','', $this->_value);
                $this->_value = preg_replace('/sizset="[^"]*"\s*/i','', $this->_value); 
                $this->_value = preg_replace('/sizcache="[^"]*"\s*/i','', $this->_value); 
                $this->_value = preg_replace('/editborder="[^"]*"\s*/i','', $this->_value);
                $this->_value = str_replace('//>', '/>', $this->_value);
                $this->_value = str_replace('\s*>', '>', $this->_value);  
                $this->_value = preg_replace('/<([^(\s|>)]*)/e', 'strtolower("\\0")', $this->_value);
                $this->_value = preg_replace('/(<img)([^>]*)(src="[^"]+)(data\/files\/[^"]+")([^>]*>)/i', '\\1\\2src="'.$path.'\\4\\5', $this->_value); 
                $this->_value = preg_replace('/(<a)([^>]*)(href="[^"]+)(data\/files\/[^"]+")([^>]*>)/i', '\\1\\2href="'.$path.'\\4\\5', $this->_value);   
                $this->_value = str_replace('<p></p>', '<p>&nbsp;</p>', $this->_value);
                $this->_value = preg_replace('/<p[^>]*><\/p>/i', '<p>&nbsp;</p>', $this->_value);  
            }
         }else{  
                $this->_value = preg_replace('/<br[^>]*><\/p>/i','</p>', $this->_value); 
                $this->_value = str_replace('<p></p>', '<p>&nbsp;</p>', $this->_value);
                $this->_value = preg_replace('/<p[^>]*><\/p>/i', '<p>&nbsp;</p>', $this->_value);   
                $config_base = Sicore_Models_Registry::getConfig(); 
                $path = Sicore_Models_Helper_FSO::joinPath($config_base->website->path);
                $this->_value = preg_replace('/(<img)([^>]*)(src="[^"]+)(data\/files\/[^"]+")([^>]*>)/i', '\\1\\2src="'.$path.'\\4\\5', $this->_value); 
                $this->_value = preg_replace('/(<a)([^>]*)(href="[^"]+)(data\/files\/[^"]+")([^>]*>)/i', '\\1\\2href="'.$path.'\\4\\5', $this->_value); 
         }
     
         return $this->_value;
     } 
     
     public function doCheckPublish(Sicore_Models_Database $node)
     {
        if(!($node instanceof Sicore_Models_Node_Page)) return false;
        
        $this->_canPublish      = $node->publishfield && $node->publishfield->value() == $this->_dbFieldName;  
        $this->_deleteSubfields = $node->publishdeletesubfields && $node->publishdeletesubfields->get($this->_dbFieldName);
        
        return $this->_canPublish ;      
     }
           
     public function canPublish()
     {
         return $this->_canPublish;
     }
     
     public function deleteSubfields()
     {
         return $this->_deleteSubfields;
     }
     
     public function field()
     {
         return $this->_dbFieldName;
     }
     
     public function alias()
     {
         return $this->_alias ? $this->_alias : $this->_dbFieldName;
     }
     
     public function __toString()
     {
         return (string)$this->value();
     }
     
     public function getFormConfig()
     {
         return  $this->_formConfig;
     }
     
     public function getConfig($key = null)
     {
         if($key){
            return isset($this->_config->$key) ?  $this->_config->$key : null;
         }
         
         return  $this->_config;
     }
     
     
     public function setConfig(Zend_Config $config)
     {
         $this->_config = $config;
     }
     
     public function setFormConfig(Zend_Config $formConfig)
     {
         $this->_formConfig = $formConfig;
         if($this->_formConfig->doNotSave || $this->_formConfig->disable){
            $this->_doNotSave = true;
         }
     }
     
     public function doNotSave(){
        return $this->_doNotSave;
     }
          
     abstract public function set($value, $rootUid = null, $canPublish = false, $deleteSubfields = false); 
         
      
}
?>
