<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @subpackage Server
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Nodes.php 1559 2010-10-22 14:05:24Z ms $
 */
class Sicore_Models_Server_Db_Nodes extends Sicore_Models_Server_Db
{
    
    /**
     * Bei den Abfragen überprüfen, ob Knoten aktiv ist (rekursiv)
     *
     * @var Boolean
     */
    protected $_doCheckIsActive  = true;
    
    /**
     * Bei den Abfragen überprüfen, ob Knoten gelöscht ist (rekursiv)
     *
     * @var Boolean
     */
    protected $_doCheckIsDeleted = true;
    
    /**
     * Bei den Abfragen überprüfen, ob Knoten eine Zeitbegrenzung hat (rekursiv)
     *
     * @var Boolean
     */
    protected $_doCheckDate      = true;
    
    /**
     * Bei den Abfragen überprüfen, ob Knoten versteckt ist (rekursiv)
     *
     * @var Boolean
     */
    protected $_doCheckIsHidden  = false;
    
    /**
     * Bei den Abfragen die Eigenschaften der Elternknoten überprüfen
     *
     * @var Boolean
     */
    protected $_doCheckParents   = true;

    /**
     * Entwurfsversion
     *
     * @var Boolean
     */
    protected $_isDraft          = false;
    
   
    
    /**
     * Sprache
     *
     * @var String
     */
    protected $_language         = 'de';  
        
    
    public function getDraft()
    {
        return $this->_isDraft;
    }
    
    public function __construct($isDraft = false){
        $this->_isDraft = (bool)$isDraft;
        parent::__construct();
    }
    
    /**
     * Überprüfen, ob Knoten aktiv ist
     *
     * @param Boolean $check Standard = true
     * @return $this
     */
    public function doCheckIsActive($check = true)
    {
        $this->_doCheckIsActive = (bool)$check;
        return $this;
    }
    
    /**
     * Überprüfen, ob Knoten gelöscht ja/nein
     *
     * @param Boolean $check Standard = true
     * @return $this Fluent Interface
     */
    public function doCheckIsDeleted($check = true)
    {
        $this->_doCheckIsDeleted = (bool)$check;
        return $this;
    }
    
    /**
     * Überprüfen, ob Knoten innerhalb Start-/Endzeit (wenn eingetragen)
     *
     * @param Boolean $check Standard = true
     * @return $this
     */
    public function doCheckDate($check = true)
    {
        $this->_doCheckDate = (bool)$check;
        return $this;
    }
    
    /**
     * Überprüfen, ob Knoten versteckt ist
     *
     * @param Boolean $check Standard = true
     * @return $this
     */
    public function doCheckIsHidden($check = false)
    {
        $this->_doCheckIsHidden = (bool)$check;
        return $this;
    }
    
    /**
     * Überprüfen, ob Knoten versteckt ist
     *
     * @param Boolean $check Standard = true
     * @return $this
     */
    public function doCheckParents($check = true)
    {
        $this->_doCheckParents = (bool)$check;
        return $this;
    }
    

    
    /**
     * Setzen der Sprachversion
     *
     * @param String $lang
     * @return $this
     */
    public function setLanguage($lang)
    {
        $this->_language = (string)$lang;
        return $this;
    }
    
    /**
     * Setzen der Entwurfsversion
     *
     * @param Boolean $isDraft
     * @return $this
     */
    public function setDraft($isDraft)
    {
        $this->_isDraft = (bool)$isDraft;
        return $this;
    }
    
    public function getUidArray($uidArray, $lang = 'de', $isDraft = false)
    {
        $pages = array();
        foreach($uidArray as $uid){
            try{
                $pages[] = $this->get($uid, $lang, $isDraft);
            }catch(Sifront_Models_Exception_Pagenotfound $e){
                continue;
            }
        }
        return $pages;
    }
    
    /**
     * Suche
     *
     * @param Integer $nid Knoten-ID
     * @param String $lang Sprachversion des Knoten
     * @param Integer $version Version des Knoten (Standard =  0)
     * @return Sicore_Models_Node
     * 
     */
    public function search($sWord, $lang = 'de')
    {
        $config_frontend     = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_FRONTEND); 
        $excludeFields       = str_replace(' ', '', $config_frontend->search->excludesFields->si_nodes);
        $excludeFields       = explode(',', $excludeFields);
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n', ''); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_');
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');     
        
        $searchFields = Sicore_Models_Registry::getTableFields('si_nodes', 'n', '', $excludeFields);
        
        $sWordexec = $this->_con->quote('%'.$sWord.'%');     
        $langQuery  = $this->_con->quote($lang);
        
        $searchFields = explode(',', $searchFields);
        $searchString  = '(1 = 0'; 
        foreach($searchFields as $searchField){
            $searchString .= " OR $searchField LIKE $sWordexec"; 
        }
        $searchString .= ')';
        
        $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields    
                FROM si_nodes AS n
                INNER JOIN si_tree AS t
                  ON n.uid = t.uid 
                LEFT JOIN si_urlalias AS url
                    ON n.uid = url.nodes_uid    
                WHERE url.version = 0 
                      AND n.version = 0 
                      AND ".$searchString." 
                      AND n.language_code = $langQuery 
                GROUP BY n.uid
                ORDER BY n.uid 
                
                ";
        
        if(!($rawNodes = $this->_con->fetchAll($sql))){  
            return false;   
        }
        $results = array();
        foreach($rawNodes as $rawNode){  
            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }  
            try{
                if($config) {
                    $node = new $classname($rawNode, $config);  
                }else{
                    $node = new $classname($rawNode);  
                }
                $parent = $this->get($node->getTree()->parent->value(), $node->language->value(), false);
                $node->addParent($parent);
                if(!$this->_checkNode($node)) continue;
                if(!$this->_checkPageAccess($node)) continue;
                if(!$this->_isSearchable($node)) continue;  
            }catch(Sifront_Models_Exception_Pageprotected $e){
                continue; 
            }catch(Sifront_Models_Exception_Pagenotfound $e){
                continue;
            }catch(Exception $e){
                continue;
            }
           
            if($node instanceof Sicore_Models_Node_Page){
                
                $data = new stdClass();
                $data->header = $node->title->value();
                $data->uid    = $node->uid->value();
                $data->text   = $text;
                $data->page   = $node;
                $data->alias  = $node->permalink->value() ? $node->permalink->value() : $node->getUrlalias()->alias->value();
                $results[$node->uid->value()] = $data;
            }else{   
                if(!($node->getParent() instanceof Sicore_Models_Node_Page)) continue;
                $text = '';
                foreach($node as $key => $property){
                    if(!is_string($property->value())) continue;
                    $text    = strip_tags($property->value());
                    $strpos1 = stripos($text, $sWord); 
                    
                    $strpos2 = stripos($property->value(), $sWord);
                    $length  = intval($config_frontend->search->result->textlen);
                    if($strpos1 !== false){    
                        $start  = $strpos1 < $length ? 0 : $strpos1 - $length;
                        $text   = substr($text, $start, strlen($sWord) + 2*$length); 
                        break; 
                    }elseif($strpos2 !== false){
                        $text =  substr($text, 0, 2*$length);
                        break;
                    }else{
                        continue; 
                    }                   
                }  
                
                $data = new stdClass();
                $data->header  = $node->getParent()->title->value();
                $data->linkUid = $node->hasSearchSingleView() ? $node->uid->value() : $node->getParent()->uid->value();
                $data->text    = str_ireplace($sWord, '<span class="si_word">'.$sWord.'</span>', $text);
                $data->page    = $node->getParent();
                $data->alias   = $node->getParent()->permalink->value() ? $node->getParent()->permalink->value() : $node->getParent()->getUrlalias()->alias->value();
                if(isset($results[$node->getParent()->uid->value()])) unset($results[$node->getParent()->uid->value()]);
                $results[$node->uid->value()] = $data;
            } 
        }                
        return $results;
    }
     
    /**
     * Einzelnen Knoten laden
     *
     * @param Integer $nid Knoten-ID
     * @param String $lang Sprachversion des Knoten
     * @param Integer $version Version des Knoten (Standard =  0)
     * @return Sicore_Models_Node
     * 
     */
    public function get($node_uid, $lang = 'de', $isDraft = false)
    {
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes', 'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree', 't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        $sqlLanguage         = $this->_con->quote($lang);
        //Node laden 
        if($isDraft || $this->_isDraft){
            $subQuery1 = self::getQueryStringSubnodes();        
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 
                                    
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields, ($subQuery1) AS subnodes_version    
                    FROM si_nodes AS n
                    INNER JOIN si_tree AS t
                      ON n.uid = t.uid
                    LEFT JOIN si_urlalias AS url
                      ON n.uid = url.nodes_uid   
                    WHERE n.uid = ? 
                          AND n.language_code = $sqlLanguage
                          AND n.version = ($subQuery2)
                          AND (url.version = ($subQuery3) OR  ISNULL(url.version))       
                    LIMIT 1";  
        }else{
            $sql  = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields    
                     FROM si_nodes AS n
                     INNER JOIN si_tree AS t
                       ON n.uid = t.uid
                     LEFT JOIN si_urlalias AS url
                       ON n.uid = url.nodes_uid   
                     WHERE n.uid = ? 
                           AND n.language_code = $sqlLanguage 
                           AND n.version = 0
                           AND (url.version <= 0 OR  ISNULL(url.version))
                     ORDER BY url.version DESC 
                     LIMIT 1";          
        }
        
        if(!($rawNode = $this->_con->fetchRow($sql, array(intval($node_uid))))){     
            throw new Sifront_Models_Exception_Pagenotfound();
        }
         
        if(strpos($rawNode->class, ':')){
            $splitName = explode(':', $rawNode->class);
            $classname = $splitName[0];
            $config    = $splitName[1];
        }else{
            $classname = $rawNode->class;
            $config    = null; 
        }
        if(!class_exists($classname)){
            Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
            throw new Sifront_Models_Exception_Pagenotfound(); 
        } 
        if($config) {
            $node = new $classname($rawNode, $config);  
        }else{
            $node = new $classname($rawNode);  
        }
        
        
        //Parents laden
        $parents   = array(); 
        $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields    
                FROM si_nodes AS n
                INNER JOIN si_tree AS t 
                    ON n.uid = t.uid
                INNER JOIN si_tree_tree AS tt
                    ON t.uid = tt.tree_uid_parent
                LEFT JOIN si_urlalias AS url
                    ON n.uid = url.nodes_uid  
                WHERE tt.tree_uid_child = ? 
                      AND n.language_code = ? 
               ";
        if($isDraft || $this->_isDraft){ 
            $sql .= " AND n.version = ($subQuery2) 
                      AND (url.version = ($subQuery3) OR ISNULL(url.version)) 
                      "; 
        }else{
            $sql .= " AND n.version = 0 
                      AND (url.version <= 0 OR  ISNULL(url.version))";  
        }
        $sql .= " ORDER BY t.level DESC";
        
        if(!($rawNodes = $this->_con->fetchAll($sql, array(intval($node->uid->value()), $lang)))){
            return $node;
        } 
          
        foreach($rawNodes as $rawNode){   
            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }
            if($config) {
                $newNode = new $classname($rawNode, $config);  
            }else{
                $newNode = new $classname($rawNode);  
            }       
            $node->addParent($newNode, $this->_doCheckParents);     
        } 
        
        if(!$this->_checkNode($node) && !($isDraft || $this->_isDraft)){
            throw new Sifront_Models_Exception_Pagenotfound(); 
        }
                     
        return $node;  
    }
    
    /**
     * Einzelnen Knoten laden
     *
     * @param Integer $uid Knoten-UID
     * @return Sicore_Models_Node
     * 
     * @todo Prüfen, ob Abfrage mit oder ohne DISTINCT performanter ist
     */
    public function getByUid($uid)
    {
        //Node laden
        $sql = "SELECT *  
                FROM si_nodes 
                WHERE uid = ?";
        if(!($rawNode = $this->_con->fetchRow($sql, array(intval($uid))))){     
            throw new Sifront_Models_Exception_Pagenotfound();
        }
        
        $isDraft =  ($rawNode->version != 0);
        
        return $this->get($rawNode->uid, $rawNode->language_code, $isDraft);
    }
    
    
    /**
     * Id des Startknoten laden
     *
     * @param String $lang Sprachversion
     * @param Integer $version Version des Knoten (Standard =  0)
     * @return Integer
     * 
     * @todo Domainabfrage für Startknoten laden
     */
    public function getRootUid($lang = 'de', $isDraft = false)
    {     
        $whereString = " 1 = 1";
        if($this->_doCheckIsActive){    
            $whereString .= " AND n.isactive = 1";      
        }
        if($this->_doCheckIsDeleted){      
            $whereString .= " AND (ISNULL( n.version_status ) OR NOT n.version_status = 'DELETED')";     
        }  
        if($this->_doCheckIsHidden){     
            $whereString .= " AND n.ishidden = 0";    
        }
        if($this->_doCheckDate){
            $whereString .= " AND (ISNULL(n.startdate) OR n.startdate <= NOW()) AND (ISNULL(n.enddate) OR n.enddate >= NOW())";
        } 
            
        $sql = "SELECT n.uid  
                FROM si_nodes AS n  
                INNER JOIN si_tree AS t 
                  ON n.uid = t.uid
               ";
        if($isDraft){
            
            $subQuery = self::getQueryStringVersionNodes();
            
            $sql .= "WHERE $whereString  
                           AND n.language_code = ?  
                           AND n.version = ($subQuery)  
                           AND t.level = 0 
                     ORDER BY t.sorting
                     LIMIT 1
                    ";
        }else{
            $sql .= "WHERE $whereString  
                           AND n.language_code = ?  
                           AND n.version = 0  
                           AND t.level = 0 
                     ORDER BY t.sorting
                     LIMIT 1
                    ";
        }  
        
        $uid =  $this->_con->fetchOne($sql, array($lang));
                           
        return intval($uid);
    }
    
    /**
     * Übergeordneten Knoten eines Inhaltselementes laden
     *
     * @param Sicore_Models_Node_Content $node Inhalts-Knoten
     * @param Integer $version Version des Knoten (Standard =  0)
     * @return Sicore_Models_Node_Content
     */
    public function getParentPage(Sicore_Models_Node_Content $node, $isDraft = false)
    {
        return $this->get($node->getTree()->parent->value(), $node->language->value(), $isDraft);
    }
    
    
    public function fetch($content = null, $limit = 10, $order = null, $inRootline = false, $tag = null, $offset = 0, $exlcudeTag = '', $uids = '', $uidsExclude = '')
    {
        $data = array();
        
        $sqlLanguage  = $this->_con->quote($this->_language);
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        if($exlcudeTags){
            $tag       = $this->_con->quote($exlcudeTags);  
            $sqlExTags = " AND NOT FIND_IN_SET($tag, n.tags)"; 
        }else{
            $sqlExTags = '';  
        }
        
        //Limit
        if(($limit = intval($limit))){
            $offset   = intval($offset);
            $sqlLimit =  $offset ? " LIMIT $offset, $limit"  : " LIMIT $limit";        
        } else{
            $sqlLimit = '';
        }
        //Node muss sich unterhalb einer UID befinden
        if(($inRootline = intval($inRootline))){ 
            $sqlInRootline = " AND tt.tree_uid_parent = $inRootline";    
        }else{
            $sqlInRootline = '';
        }
        //Sortierung
        if($order){
            if($order == 'RAND()' || $order == 'RAND'){
                $sqlOrder = "ORDER BY RAND()";
            }elseif(substr($order, 0, 7)  == 'sorting'){
                 $sqlOrder = 'ORDER BY t.'.$order;
            }else{    
                switch(substr($order, 0, 2)){
                    case 'n.':
                    case 't.':
                        $sqlOrder = "ORDER BY $order";
                        break;
                    default:
                        $sqlOrder = "ORDER BY n.$order";
                        break;    
                } 
            } 
        }else{
            $sqlOrder = "";
        }
        //Abzufragende Objekte
        if(empty($content)){
            $class  = 'Sicore_Models_Node_Page';
        }else{
            $content = str_replace("'", '', $content);
            switch(substr($content, 0, 1)){
                case '_':
                case ':':
                   $object = "Sicore_Models_Node_Content$content";
                   break;
                default:
                   $object = "Sicore_Models_Node_Content:$content";
                   break;   
            }
        }
        
        //Tags
        if($tag){
            $tag     = $this->_con->quote($tag);  
            $sqlTags = " AND FIND_IN_SET($tag, n.tags)";    
        } else{
            $sqlTags = '';
        }
        
        //UID-Abfrage
        if($uids){
            $uidArr  = explode(',', $uids);
            $sqlUids = " AND n.uid IN($uids)";
        }
        if($uidsExclude){
            $sqlUidsEx = " AND NOT n.uid IN($uidsExclude)";  
        }
        if($this->_isDraft){ 
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 
            
            $sql = "SELECT DISTINCT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    INNER JOIN si_tree_tree AS tt
                        ON tt.tree_uid_child = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE n.language_code = ".$sqlLanguage." 
                          AND n.class = '$object' 
                          AND n.version = ($subQuery2) 
                          AND n.ishidden = 0   
                          AND n.isactive = 1  
                          AND (url.version = ($subQuery3) OR ISNULL(url.version))
                          AND (version_status != 'DELETED' OR ISNULL(version_status))  
                          $sqlTags
                          $sqlExTags
                          $sqlInRootline
                          $sqlUids
                          $sqlUidsEx
                    $sqlOrder
                    $sqlLimit
                    ";    
        }else{
            $sql = "SELECT DISTINCT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    INNER JOIN si_tree_tree AS tt
                        ON tt.tree_uid_child = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE n.language_code = ".$sqlLanguage." 
                          AND n.class = '$object'
                          AND n.version = 0
                          AND n.ishidden = 0   
                          AND n.isactive = 1
                          AND (url.version = 0 OR ISNULL(url.version))
                          $sqlTags
                          $sqlExTags
                          $sqlInRootline
                          $sqlUids
                          $sqlUidsEx
                    $sqlOrder
                    $sqlLimit
                    ";
        }
        
        if(!($rawNodes = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $nodes = array();
        foreach($rawNodes as $rawNode){
            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }  
            if($config){
                $node  = new $classname($rawNode, $config);        
            }else{
               $node  = new $classname($rawNode); 
            }
            
            
            if(is_array($uidArr) && array_search($node->uid->value(), $uidArr) !== false){
                $nodes[array_search($node->uid->value(), $uidArr)]  = $node; 
            }else{
                $nodes[]  = $node;
            }
        } 
        if(is_array($uidArr)){
            ksort($nodes);
        }            
        return $nodes;
    }
    
    /**
     * Alle Knoten unterhalb einer bestimmten Knoten-Id laden
     *
     * @param Integer $uid übergeordnete Knoten-Id 
     * @param array $includeClasses nur bestimmte Klassen (auch abgeleitete) laden
     * @param array $excludeClasses bestimmte Klassen (auch abgeleitete) nicht laden
     * @return array
     */
    public function getByParentUid($uid, array $includeClasses = array(), array $excludeClasses = array(), $isDraft = false, $limit = 0, $onlyNotPublished = false, $onlySingeViews = false)
    {        
        $sqlUidparent = $this->_getQueryStringUidParent($uid);  
        $sqlLanguage  = $this->_con->quote($this->_language);
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
                
        //Aktuelle Ebene laden  
        if($this->_isDraft || $isDraft){
            if($onlyNotPublished){
                $subQuery1 = self::getQueryStringSubnodes(); 
                $subQuery2 = 1;
                $subQuery3 = self::getQueryStringVersionUrlalias();
                $queryPart = "AND (n.version = 1 OR ($subQuery1) = 1)";      
            }else{
                $subQuery1 = self::getQueryStringSubnodes();        
                $subQuery2 = self::getQueryStringVersionNodes(); 
                $subQuery3 = self::getQueryStringVersionUrlalias();
                $queryPart = "AND n.version = ($subQuery2)"; 
            }
                 
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields, ($subQuery1) AS  subnodes_version      
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid 
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  ".$sqlUidparent." 
                           AND n.language_code = ".$sqlLanguage." 
                           $queryPart 
                           AND (url.version =($subQuery3) OR ISNULL(url.version)) 
                    ORDER BY t.sorting
                   ";  
        }else{
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  ".$sqlUidparent."  
                           AND n.language_code = ".$sqlLanguage." 
                           AND n.version = 0 
                           AND (url.version <= 0 OR ISNULL(url.version) )
                    ORDER BY t.sorting, url.version DESC
                    ";
        }     
        
        if(!($rawNodes = $this->_con->fetchAll($sql))){
            return array();
        }
        //Sichtbarbeit Oberseiten abfragen
        if($this->_doCheckParents && $rawNodes[0]->tree_level > 0){
            try{
                $currentNode = $this->get($uid, $this->_language, $this->_isDraft || $isDraft);

            }catch(Sifront_Models_Exception_Pagenotfound $e){  
                return array();
            }
        }    
        $nidExists = array();
        
        $nodes   = array();
        $counter = 1;
        
        foreach($rawNodes as $rawNode){
            if(in_array($rawNode->uid, $nidExists)) continue;

            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }  
            if($config){
                $node  = new $classname($rawNode, $config);        
            }else{
               $node  = new $classname($rawNode); 
            }
            if($onlySingeViews && !$node->hasSingleView()) continue;
            
            if($this->_doCheckParents && $rawNodes[0]->tree_level > 0){
                $node->addParent($currentNode);
            }      
            $nidExists[] = $node->uid->value();
            
            if(count($includeClasses) && !$this->_checkNodeClass($node,  $includeClasses)){
                unset($node);
                continue;    
            } 
            if(count($excludeClasses) &&  $this->_checkNodeClass($node,  $excludeClasses)){
                unset($node);
                continue; 
            }
            if(!$this->_checkNode($node)) continue;
            
            if(intval($limit) && $counter > $limit) break;
            $nodes[$node->getTree()->sorting->value()] = $node;
            $counter++;
        }
        
        ksort($nodes); 
                        
        return $nodes;
    }
    
    public function getPageSorting($uid, $language = 'de', $isDraft = false)
    {
        $sqlUidparent = $this->_getQueryStringUidParent($uid);
        $sqlLanguage  = $this->_con->quote($this->_language);
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        //Aktuelle Ebene laden  
        if($this->_isDraft || $isDraft){
            $subQuery1 = self::getQueryStringSubnodes();        
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 
            
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields, ($subQuery1) AS  subnodes_version      
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid 
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  ".$sqlUidparent." 
                           AND n.language_code = ".$sqlLanguage." 
                           AND n.version = ($subQuery2) 
                           AND (url.version =($subQuery3) OR ISNULL(url.version)) 
                    ORDER BY t.sorting
                   ";  
        }else{
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  ".$sqlUidparent."  
                           AND n.language_code = ".$sqlLanguage." 
                           AND n.version = 0  
                           AND (url.version = 0 OR ISNULL(url.version))
                    ORDER BY t.sorting
                    ";
        }     
        
        if(!($rawNodes = $this->_con->fetchAll($sql))){
            return array();
        }
        
        $sorting = array();
        foreach($rawNodes as $node){
            $sorting[$node->uid] = $node->tree_sorting;
        }                 
        return $sorting;
    }
    
    public function getByTag($tag, $language = 'de', $isDraft = false)
    {        
        $sqlLanguage  = $this->_con->quote($this->_language);
        $sqltag       = $this->_con->quote($tag); 
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        //Aktuelle Ebene laden  
        if($this->_isDraft || $isDraft){
            $subQuery1 = self::getQueryStringSubnodes();        
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 

            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields, ($subQuery1) AS  subnodes_version      
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid 
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE FIND_IN_SET(".$sqltag.", n.tags) 
                          AND n.language_code = $sqlLanguage 
                          AND n.version = ($subQuery2) 
                          AND (url.version =($subQuery3) OR ISNULL(url.version)) 
                    ORDER BY t.sorting
                   ";  
        }else{
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE FIND_IN_SET(".$sqltag.", n.tags)   
                          AND n.language_code = $sqlLanguage 
                          AND n.version = 0  
                          AND (url.version = 0 OR ISNULL(url.version))
                    ORDER BY t.sorting
                    ";
        }     
         
        if(!($rawNodes = $this->_con->fetchAll($sql))){
            return array();
        }
           
        $nidExists = array();
        
        $nodes = array();

        foreach($rawNodes as $rawNode){
            if(in_array($rawNode->uid, $nidExists)) continue;  

            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }
            if($config) {
                $node = new $classname($rawNode, $config);  
            }else{
                $node = new $classname($rawNode);  
            }
                 
            $nidExists[] = $node->uid->value();
            
            
            if(!$this->_checkNode($node)) continue;
            $nodes[] = $node;
        }
                        
        return $nodes;
    }
    
    public function saveSorting($uid, $position)
    {
        $this->_con->update('si_tree', array('sorting' => $position), ' uid = '.intval($uid));
    }
    
    public function getFirstSub($uid, $lang, $isDraft = false){
        return $this->getSub($uid, $lang, $isDraft, false); 
    }
    
    public function getRandomSub($uid, $lang, $isDraft = false){
        return $this->getSub($uid, $lang, $isDraft, true); 
    }
    
    public function getSub($uid, $lang, $isDraft = false, $getRandom = false)
    {        
        $sqlUidparent = $this->_getQueryStringUidParent($uid);
        $sqlLanguage  = $this->_con->quote($this->_language);
        
        $si_nodes_fields     = Sicore_Models_Registry::getTableFields('si_nodes',    'n'); 
        $si_tree_fields      = Sicore_Models_Registry::getTableFields('si_tree',     't', 'tree_'); 
        $si_urlalias_fields  = Sicore_Models_Registry::getTableFields('si_urlalias', 'url', 'urlalias_');
        
        //Aktuelle Ebene laden  
        if($this->_isDraft || $isDraft){
            $subQuery1 = self::getQueryStringSubnodes();        
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 
            
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields, ($subQuery1) AS  subnodes_version      
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid 
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  ".$sqlUidparent." 
                           AND t.ispage = 1  
                           AND n.language_code = ".$sqlLanguage." 
                           AND n.version = ($subQuery2) 
                           AND (url.version =($subQuery3) OR ISNULL(url.version))"; 
            $sql .= $getRandom ? " ORDER BY RAND()" : " ORDER BY t.sorting"; 
            $sql .= " LIMIT 1";  
        }else{
            $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields 
                    FROM si_nodes AS n 
                    INNER JOIN si_tree AS t 
                        ON n.uid = t.uid
                    LEFT JOIN si_urlalias AS url
                        ON n.uid = url.nodes_uid  
                    WHERE  t.ispage = 1  
                           AND ".$sqlUidparent."  
                           AND n.language_code = ".$sqlLanguage."
                           AND n.version = 0  
                           AND (url.version = 0 OR ISNULL(url.version))
                    "; 
            $sql .= $getRandom ? " ORDER BY RAND()" : " ORDER BY t.sorting"; 
            $sql .= " LIMIT 1";
         
        }     
        
        if(!($rawNode = $this->_con->fetchRow($sql))){
            return null;
        }
        
        if(strpos($rawNode->class, ':')){
            $splitName = explode(':', $rawNode->class);
            $classname = $splitName[0];
            $config    = $splitName[1];
        }else{
            $classname = $rawNode->class;
            $config    = null; 
        }
        if(!class_exists($classname)){
            Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
            return null;
        }
        if($config) {
            $node = new $classname($rawNode, $config);  
        }else{
            $node = new $classname($rawNode);  
        }
        $parents   = array();
        
        $sql = "SELECT $si_nodes_fields, $si_tree_fields, $si_urlalias_fields    
                FROM si_nodes AS n
                INNER JOIN si_tree AS t 
                    ON n.uid = t.uid
                INNER JOIN si_tree_tree AS tt
                    ON t.uid = tt.tree_uid_parent
                LEFT JOIN si_urlalias AS url
                    ON n.uid = url.nodes_uid  
                WHERE tt.tree_uid_child = ? AND
                      n.language_code = ? 
               ";
        
        if($isDraft || $this->_isDraft){ 
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $subQuery3 = self::getQueryStringVersionUrlalias(); 
            
            $sql .= " AND n.version = ($subQuery2) 
                      AND (url.version = ($subQuery3) OR ISNULL(url.version)) 
                      ";  
         
        }else{
            $sql .= " AND n.version = 0 
                      AND (url.version = 0 OR ISNULL(url.version))";  
        }
        $sql .= " ORDER BY t.level DESC";
        
        
        if(!($rawNodes = $this->_con->fetchAll($sql, array(intval($node->uid->value()), $lang)))){
            return $node;
        } 
          
        foreach($rawNodes as $rawNode){   
            if(strpos($rawNode->class, ':')){
                $splitName = explode(':', $rawNode->class);
                $classname = $splitName[0];
                $config    = $splitName[1];
            }else{
                $classname = $rawNode->class;
                $config    = null; 
            }
            if(!class_exists($classname)){
                Sicore_Models_logger::error("Klasse '$classname' existiert nicht.");
                continue;
            }
            if($config) {
                $newNode = new $classname($rawNode, $config);  
            }else{
                $newNode = new $classname($rawNode);  
            }
            $node->addParent($newNode, $this->_doCheckParents);     
        } 
        
        if(!$this->_checkNode($node) && !($isDraft || $this->_isDraft)){
            throw new Sifront_Models_Exception_Pagenotfound(); 
        }
                    
        return $node;  
    }
    
    /**
     * Klasse eines Knoten überprüfen
     *
     * @param Sicore_Models_Node $node zu überprüfender Knoten 
     * @param String $classNames Name der zu überprüfenden Klasse
     * @return Boolean
     */
    protected function _checkNodeClass($node, array $classNames)
    {
        foreach($classNames as $className){
            if($node instanceof $className) return true;
        }
        
        return false;
    }
    
    /**
     * Zugriff auf eine Knoten prüfen
     *
     * @param Sicore_Models_Node $node zu überprüfender Knoten 
     * @return Boolean
     */
    protected function _checkPageAccess(Sicore_Models_Node $node)
    {
        
        if(!($accessList = $node->access->toArray())){
            return true;
        }
        if(!Zend_Auth::getInstance()->hasIdentity()){
            return false;   
        }
        $user = Zend_Auth::getInstance()->getIdentity();
        if($user->isadmin->value()){
            return true;  
        }     
        foreach($accessList as $role){
            if(Sicore_Models_Registry::getAcl()->hasRole($role)){
                return true;
            }
        }
  
        return false;
    }
    
    /**
     * Knoten darf gesucht werden oder nicht
     *
     * @param Sicore_Models_Node $node zu überprüfender Knoten 
     * @return Boolean
     */
    protected function _isSearchable(Sicore_Models_Node $node)
    {
        return !$node->nosearch->value();
    } 
    
    /**
     * Sichtbarkeit eines Knoten prüfen
     *
     * @param Sicore_Models_Node $node zu überprüfender Knoten 
     * @return Boolean
     */
    protected function _checkNode(Sicore_Models_Node $node)
    {
        if($this->_doCheckIsActive && !$node->isActive()){
            return false;       
        }
        
        if($this->_doCheckIsDeleted && $node->version_status->value() == 'DELETED'){
            return false;       
        }
        
        if($this->_doCheckIsHidden && $node->isHidden()){
            return false;       
        }
       
        if($this->_doCheckDate && $node->startdate->value() instanceof Zend_Date){
            if($node->startdate->value()->isLater(new Zend_Date())){
                return false;
            }
        }
        
        if($this->_doCheckDate && $node->enddate->value() instanceof Zend_Date){
            if($node->enddate->value()->isEarlier(new Zend_Date())){
                return false;
            }
        }
        
        return true;
    }
    
    /**
     * Ändert den Namen eines Knoten
     *
     * @param Sicore_Models_Node $node
     * @param String $newName  
     */
    public function rename(Sicore_Models_Node $node, $newName)
    {
        
        if(empty($newName)){
            return false;
        }
        
        if($node->title->value() == $newName){
            return true;
        }
        
        $this->_con->beginTransaction();
        try{
            $langQuery  = $this->_con->quote($node->language->value());
            
            $sql = "DELETE FROM si_nodes
                    WHERE uid = ".intval($node->uid->value())."
                          AND language_code = $langQuery
                          AND version = 1";
            $this->_con->exec($sql);
                          
            //Neue Version des Inhaltsknoten einfügen
            if($node->version_status->value() != 'ADDED' && $node->version_status->value() != 'DELETED'){
                $node->version_status->set('EDITED');
                $node->version->set(1);    
            }  
            $node->title->set($newName);
            $this->insert($node);
            //si_urlalias anpassen
            $urlaliasCurrent = $node->getUrlalias()->alias->value();
            if(($level = $node->getTree()->level->value())){
                $parentNode = $node->getParent($level - 1);
                $urlaliasRoot = $parentNode->getUrlalias()->alias->value();                
            }
            
            $mUrlalias = new Sicore_Models_Server_Db_Urlalias();
            if(($urlalias = $mUrlalias->check($urlaliasRoot, $newName, $node->uid->value()))){
                $urlalias            = $this->_con->quote($urlalias);   
                $urlaliasCurrentSQL1 = $this->_con->quote($urlaliasCurrent);
                $urlaliasCurrentSQL2 = $this->_con->quote($urlaliasCurrent.'%'); 
                
                if($node->version_status->value()){
                    $sql = "UPDATE si_urlalias
                            SET alias = REPLACE(alias, $urlaliasCurrentSQL1, $urlalias)
                            WHERE alias LIKE $urlaliasCurrentSQL2";
                    $this->_con->exec($sql); 
                }else{ 
                    $sql = "INSERT INTO si_urlalias
                               (alias, language_code, nodes_uid, version)  
                            SELECT REPLACE(alias, $urlaliasCurrentSQL1, $urlalias), language_code, nodes_uid, 1
                            FROM si_urlalias 
                            WHERE alias LIKE $urlaliasCurrentSQL2";
                    $this->_con->exec($sql);
                }           
            }
            $this->_con->commit();
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e); 
            $this->_con->rollback();
            return false;
        }
        
        return true;
    }
    
    public function publishProperty($node, $property)
    {
        if(!($node instanceof Sicore_Models_Node)){ 
            $node   = $this->setDraft(true)
                             ->doCheckParents(false) 
                             ->doCheckIsDeleted(false)
                             ->doCheckIsActive(false)
                             ->doCheckDate(false)
                             ->getByUid(intval($node));
        } 
        
        $data[$node->$property->field()] = $node->$property->value(); 
        $data['publishfield']            = new Zend_Db_Expr('NULL');   
        $data['publishdeletesubfields']  = new Zend_Db_Expr('NULL');
        
        $this->_con->update('si_nodes', $data, 'uid = '.$node->uid->value().' AND version = 0');
        /*
        $sql = "DELETE FROM si_version 
                WHERE uid IN (
                    SELECT version_uid 
                    FROM si_nodes
                    WHERE uid = ".$node->uid->value()." AND publishfield = '".$node->$property->field()."' 
                ) AND uid > 0
                ";
        $this->_con->exec($sql);
        */   
    }
    
    public function deleteSubfields($uid, $field, $isDraft = true)
    {
        $sqlVersion = $isDraft ? " AND version > 0 " : "";
        

        $sql = "UPDATE si_nodes AS n
                INNER JOIN si_tree AS t
                    ON n.uid = t.uid 
                SET n.$field = '' 
                WHERE  t.ispage = 1 AND
                       n.uid IN (
                         SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($uid)."
                       )
                       $sqlVersion";

        $this->_con->exec($sql);        
    }
    
    /**
     * Ändert den Namen eines Knoten
     *
     * @param Sicore_Models_Node $node
     * @param String $newName  
     */
    public function update(Sicore_Models_Node $node)
    {
        if(!($oldNode = $this->get($node->uid->value(), $node->language->value(), true))){
            return false;
        }
                
        $this->_con->beginTransaction();
        try{                  
            $urlaliasRoot = '';
            
            //Knoten speichern
            $sqlLanguage  = $this->_con->quote($node->language->value());
            $sql = "DELETE FROM si_nodes
                    WHERE uid = ".intval($node->uid->value())."
                          AND language_code = $sqlLanguage 
                          AND version = 1 ";
            $this->_con->exec($sql);
            
            if($node->version_status->value() != 'ADDED' && $node->version_status->value() != 'DELETED'){
                $node->version_status->set('EDITED');
                $node->version->set(1);   
            }
            $this->insert($node, false);
            
            if($oldNode instanceof Sicore_Models_Node_Page && $node instanceof Sicore_Models_Node_Page){
                if($oldNode->title->value() != $node->title->value()){
                    $urlaliasCurrent = $oldNode->getUrlalias()->alias->value();
                    if(($level = $oldNode->getTree()->level->value())){
                        $parentNode   = $oldNode->getParent($level - 1);
                        $urlaliasRoot = $parentNode->getUrlalias()->alias->value();                
                    }else{
                        $urlaliasRoot = '';
                    }

                    $mUrlalias = new Sicore_Models_Server_Db_Urlalias();
                    $urlalias = $mUrlalias->check($urlaliasRoot, $node->title->value());            
                    
                    $urlalias            = $this->_con->quote($urlalias);   
                    $urlaliasCurrentSQL1 = $this->_con->quote($urlaliasCurrent);
                    $urlaliasCurrentSQL2 = $this->_con->quote($urlaliasCurrent.'%');          
                    
                    if($node->version_status->value() == 'ADDED'){
                        $sql = "UPDATE IGNORE si_urlalias
                                SET alias = REPLACE(alias, $urlaliasCurrentSQL1, $urlalias)
                                WHERE alias LIKE $urlaliasCurrentSQL2 AND 
                                     (nodes_uid = ".intval($node->uid->value())."
                                       OR nodes_uid IN(SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value())." )
                                      )";
                        $this->_con->exec($sql); 
                    }elseif(!empty($urlaliasCurrent)){  
                        $sql = "REPLACE INTO si_urlalias
                                   (alias, language_code, nodes_uid, version)  
                                SELECT REPLACE(alias, $urlaliasCurrentSQL1, $urlalias), language_code, nodes_uid, 1
                                FROM si_urlalias 
                                WHERE alias LIKE $urlaliasCurrentSQL2 AND
                                      (nodes_uid = ".intval($node->uid->value())."
                                       OR nodes_uid IN(SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value())." )
                                      )";
                        $this->_con->exec($sql);
                    }

                }
            }
            $this->_con->commit();
        }catch(Exception $e){
            Sicore_Models_Logger::exception($e); 
            $this->_con->rollback();
            return false;
        } 
        return true;
    }
    
    /**
     * Erstellt einen Knoten
     *
     * @param Sicore_Models_Node $nodeDest Referenzknoten, nach dem der neue eingefügt wird
     * @param String $name Name des neuen Knoten  
     * @param String $mode Position des neuen Knotens "after", oder "top" für Unterseite
     * @param String $class interne Klasse des neuen Knoten
     * @param String $flag Benutzerflag, um Tabellen zu "sperren" 
     */
    public function create(Sicore_Models_Node $nodeDest, $data, $mode, $class, $flag, $lang = 'de') 
    {
        if($data instanceof Sicore_Models_Node_Page){
            $name = $data->title->value();
        }else{
            $name = $data;
        }
        
        $this->_con->beginTransaction();
        try{
            
            if($mode == 'after'){
                //Neue Seite
                $sortingNew = $nodeDest->getTree()->sorting->value() + 1;
                $newLevel   = $nodeDest->getTree()->level->value(); 
                $parent     = $nodeDest->getTree()->parent->value();
            }elseif($mode == 'inside'){
                //Neue Unterseite  
                $sortingNew = 1;
                $newLevel   = $nodeDest->getTree()->level->value()+1; 
                $parent     = $nodeDest->uid->value();
            }else{
                $this->_con->rollback();  
                return false;
            }
            
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = sorting + 1 
                    WHERE ispage = 1 
                          AND uid_parent = ".intval($parent)."
                          AND sorting >= ".intval($sortingNew)."
                    ORDER BY sorting DESC";
            $this->_con->exec($sql);
            
            //Neuen Knoten erstellen
            $sql = "INSERT INTO si_tree
                       ( uid_parent, sorting, level, ispage)
                    VALUES(".intval($parent).", ".intval($sortingNew).", ".intval($newLevel).", 1)
                    ";       
             $this->_con->exec($sql); 
             $newId = $this->_con->lastInsertId();
             
             //Tree_Tree ergänzen
             $rootline = $nodeDest->getAllParents();
             foreach($rootline as $level => $node){
                 $sql = "INSERT INTO si_tree_tree 
                           (tree_uid_child, tree_uid_parent)
                         VALUES
                           (".intval($newId).", ".$this->_con->quote($node->uid->value()).")
                         ";  
                 $this->_con->exec($sql);   
                 $rootPermalink = $node->permalink->value();
                 $rootAlias     = $node->getUrlalias()->alias->value();
             }
             if($mode == 'inside'){
                $sql = "INSERT INTO si_tree_tree 
                           (tree_uid_child, tree_uid_parent)
                         VALUES
                           (".intval($newId).", ".$this->_con->quote($nodeDest->uid->value()).")
                         ";  
                 $this->_con->exec($sql); 
                 
                 $rootPermalink = $nodeDest->permalink->value();
                 $rootAlias     = $nodeDest->getUrlalias()->alias->value();
             }   
             //si_urlalias
             $mUrlalias = new Sicore_Models_Server_Db_Urlalias();
             $newAlias = $mUrlalias->check($rootAlias, $name);
             $sql = "INSERT INTO si_urlalias
                       (alias, language_code, nodes_uid, version)  
                     VALUES
                       (".$this->_con->quote($newAlias).", ".$this->_con->quote($nodeDest->language->value()).", ".intval($newId).", 1)
                    "; 
             $this->_con->exec($sql); 
             
             //Knoteninhalt einfügen
             if($data instanceof Sicore_Models_Node_Page){
                 $data->uid->set(intval($newId));
                 $data->version->set(1);
                 $data->version_status->set('ADDED');
                 $data->language->set($lang);
                 $data->isActive->set(true);  
                 $data->added->set(new Zend_Db_Expr('NOW()'));
                 $data->changed->set(new Zend_Db_Expr('NOW()'));
                 if($rootPermalink){
                    $newPermalink = $mUrlalias->check($rootPermalink, $name);
                    $data->permalink->set($newPermalink); 
                 }
                 $this->insert($data);
             }else{
                 $mNode = new Sicore_Models_Node_Page();
                 $mNode->uid->set(intval($newId));
                 $mNode->version->set(1);
                 $mNode->version_status->set('ADDED');
                 $mNode->class->set($class);
                 $mNode->added->set(new Zend_Db_Expr('NOW()'));
                 $mNode->changed->set(new Zend_Db_Expr('NOW()'));
                 $mNode->language->set($lang);
                 $mNode->isActive->set(true);  
                 $mNode->title->set($name);
                 if($rootPermalink){
                    $newPermalink = $mUrlalias->check($rootPermalink, $name);
                    $mNode->permalink->set($newPermalink); 
                 }
                 $this->insert($mNode);
             }
             $this->_con->commit(); 
             return $newId;
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            die();
        }
    }
    
    
    /**
     * Erstellt einen Knoten
     *
     * @param Sicore_Models_Node_Content $content Neuer Inhaltsknoten
     * @param Sicore_Models_Node_Page $page Seite  
     * @param String $flag Benutzerflag, um Tabellen zu "sperren"
     * @param Sicore_Models_Node_Content $nodeDest Inhalt, nach dem der neue Inhalt eingefügt werden soll 
     */
    public function createContent(Sicore_Models_Node_Content $content, Sicore_Models_Node_Page $page, $flag, $nodeDest = null, $position = 'after', $lang = 'de') 
    {
        $this->_con->beginTransaction();
        try{
     
            if($nodeDest instanceof Sicore_Models_Node_Content){
                if($page->uid->value() != $nodeDest->getTree()->parent->value()) throw new Exception('$node kein Inhalt der Seite.');                    
                switch($position){
                    case 'before':
                        $sortingNew = $nodeDest->getTree()->sorting->value();
                        break;
                    default:
                        $sortingNew = $nodeDest->getTree()->sorting->value() + 1;
                        break;
                }             
            }else{
                $sortingNew = 1;
            }
            $newLevel   = $page->getTree()->level->value() + 1; 
            $parent     = $page->uid->value();
            
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = sorting + 1 
                    WHERE ispage = 0 
                          AND uid_parent = ".intval($parent)."
                          AND sorting >= ".intval($sortingNew)."
                    ORDER BY sorting DESC";
            $this->_con->exec($sql);
            
            //Neuen Knoten erstellen
            $sql = "INSERT INTO si_tree
                       (uid_parent, sorting, level, ispage)
                    VALUES(".intval($parent).", ".intval($sortingNew).", ".intval($newLevel).", 0)
                    ";       
             $this->_con->exec($sql); 
             $newId = $this->_con->lastInsertId();
             //Tree_Tree ergänzen
             $rootAlias = '';
             if($nodeDest){
                 $rootline = $nodeDest->getAllParents();
                 foreach($rootline as $level => $node){
                     $sql = "INSERT INTO si_tree_tree 
                               (tree_uid_child, tree_uid_parent)
                             VALUES
                               (".intval($newId).", ".$this->_con->quote($node->uid->value()).")
                             ";  
                     $this->_con->exec($sql);
                     
                     $rootPermalink = $node->permalink->value();
                     $rootAlias     = $node->getUrlalias()->alias->value();
                 }
                 
             }else{
                 $rootline = $page->getAllParents();
                 foreach($rootline as $level => $node){
                     $sql = "INSERT INTO si_tree_tree 
                               (tree_uid_child, tree_uid_parent)
                             VALUES
                               (".intval($newId).", ".$this->_con->quote($node->uid->value()).")
                             ";  
                     $this->_con->exec($sql);
                     
                     $rootPermalink = $node->permalink->value();
                     $rootAlias     = $node->getUrlalias()->alias->value();
                 }
                 $sql = "INSERT INTO si_tree_tree 
                           (tree_uid_child, tree_uid_parent)
                         VALUES
                           (".intval($newId).", ".$this->_con->quote($page->uid->value()).")
                         ";  
                 $this->_con->exec($sql); 
                 
                 $rootPermalink = $page->permalink->value();
                 $rootAlias     = $page->getUrlalias()->alias->value();
             } 
             
             //si_urlalias TODO  
             $mUrlalias = new Sicore_Models_Server_Db_Urlalias();
             $newAlias = $mUrlalias->check($rootAlias, $newId);
             $sql = "INSERT INTO si_urlalias
                       (alias, language_code, nodes_uid, version)  
                     VALUES
                       (".$this->_con->quote($newAlias).", ".$this->_con->quote($page->language->value()).", ".intval($newId).", '1')
                    ";  
             $this->_con->exec($sql); 
             
             //Knoteninhalt einfügen
             $content->uid->set(intval($newId));
             $content->version->set(1);
             $content->version_status->set('ADDED');
             $content->language->set($lang);
             $content->isActive->set(true);  
             $content->added->set(new Zend_Db_Expr('NOW()'));
             $content->changed->set(new Zend_Db_Expr('NOW()'));
             $this->insert($content);
             $this->_con->commit(); 
             
             //Urlalias setzen
             $alias = new Sicore_Models_Node_Urlalias();
             $alias->alias->set($newAlias);  
             $content->setUrlalias($alias); 
             return $content;
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            throw new Exception($e->__toString());
        }
    }
    
    /**
     * Entfernt einen Knoten
     *
     * @param Sicore_Models_Node $node
     * @return Boolean Knoten konnte gelöscht werden (ja/nein)  
     */
    public function remove(Sicore_Models_Node $node) 
    {
        $this->_con->beginTransaction(); 
        
        try{
            $si_nodes_fields1  = Sicore_Models_Registry::getTableFields('si_nodes', ''); 
            
            $si_nodes_fields2  = str_replace(' ', '', $si_nodes_fields1);
            $si_nodes_fields2  = str_replace(',version_status,', ",'DELETED',", $si_nodes_fields2);
            $si_nodes_fields2  = str_replace(',version,',        ",'1',",       $si_nodes_fields2);
             
            $sql = "INSERT INTO si_nodes ($si_nodes_fields1)
                      SELECT $si_nodes_fields2
                      FROM si_nodes 
                      WHERE uid = ".intval($node->uid->value())."
                    ON DUPLICATE KEY UPDATE 
                      version = 1, 
                      version_status = 'DELETED'";
            $this->_con->exec($sql);
            $this->_con->commit(); 
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            die();
            return false;
        }
        
        return false;
    }
    
    /**
     * Kiopiert einen Knoten
     *
     * @param Sicore_Models_Node $nodeDest   
     * @param String $type 'before', 'after' oder 'inside
     */
    public function copyContent(Sicore_Models_Node $dest, $mode, $uids)
    {
        switch($mode){
            case 'after':
            case 'inside':  
                krsort($uids); 
                break;
            default:
                return false;
        }
        foreach($uids as $key => $entry){
            if($entry instanceof Sicore_Models_Node_Content){
                $content = $entry;    
            }else{
                try{
                    $content = $this->get($entry['uid'], 'de', true);
                }catch(Exception $e){
                    Sicore_Models_Clipboard::deleteContent($key);
                    continue;
                }
            }
            if($dest instanceof Sicore_Models_Node_Page){
               $this->createContent($content, $dest, null, null, 'inside'); 
            }else{
                $page = $dest->getParent();
               $this->createContent($content, $page, null, $dest, 'after'); 
            } 
        }
        
        return true;
    }
    
    /**
     * Kiopiert einen Knoten
     *
     * @param Sicore_Models_Node $nodeDest   
     * @param String $type 'before', 'after' oder 'inside
     */
    public function copyPages(Sicore_Models_Node_Page $dest, $mode, $uids, $depthRemain = 0)
    { 
     
        switch($mode){
            case 'inside': 
            case 'after':
                krsort($uids); 
                break;
            default:
                return false;
        }
        
        foreach($uids as $key => $entry){
            
            if($entry instanceof Sicore_Models_Node_Page){
                $page = $entry;
                $uid  = $page->uid->value();    
            }else{
                try{
                    $page = $this->get($entry['uid'], 'de', true);
                    $uid  = $page->uid->value();
                }catch(Exception $e){
                    Sicore_Models_Clipboard::deleteContent($key);
                    continue;
                }
            } 
                        
            $newUid = $this->create($dest, $page, $mode, null);
            //Inhalt
            $contents = $this->getByParentUid($uid, array('Sicore_Models_Node_Content'), array(), true);
            $newPage  = $this->getByUid($newUid);
            $this->copyContent($newPage, 'inside', $contents);
            //Unterseiten
            if($depthRemain != 0){
                $subpages = $this->getByParentUid($uid, array('Sicore_Models_Node_Page'), array(), true);
                if(count($subpages)){
                    $this->copyPages($newPage, 'inside', $subpages, $depthRemain - 1);
                } 
            }
        }
        
        return true;
    }
    
    /**
     * Kiopiert einen Knoten
     *
     * @param Sicore_Models_Node $nodeDest   
     * @param String $type 'before', 'after' oder 'inside
     */
    protected function _copy(Sicore_Models_Node $dest, $mode, $uids, $isPage)
    {
        $isPage   = intval($isPage);
        $elements = count($uids);
        
        if(!$elements) return false;
        
        if($mode == 'inside'){
            $uidParent = $dest->uid->value();
            $position  = 1;
        }elseif($mode == 'after'){
            $uidParent = $dest->getTree()->parent->value();
            $position  = $dest->getTree()->sorting->value() + 1;
        }else{
            return false;
        }
        
        foreach($uids as $uid){
            $content = $this->getByUid($uid);
            if(!$uid) throw new Exception();
            
            $tableName = 'si_temp'.rand();
            $sql = "CREATE TABLE $tableName ENGINE=MEMORY SELECT * FROM si_tree_tree WHERE 1=0";
            $this->_con->exec($sql);
            
            $sql = "INSERT INTO si_nodes FROM si_nodes"; 
        }

        return true;
    }
    
    public function updateContentSorting(array $nodes, $uid_parent)
    {
        $this->_con->beginTransaction();
        try{
            $parentRand = -rand(1,10000);
            $sql = "UPDATE si_tree
                    SET uid_parent   = ".$parentRand."
                    WHERE ispage = 0 
                          AND uid_parent = ".intval($uid_parent);
            $this->_con->exec($sql);
            foreach($nodes as $node){
                if($node->getTree()->parent->value() != $uid_parent) throw Exception();
                
                $sql = "UPDATE si_tree
                        SET sorting    = ".intval($node->getTree()->sorting->value()).",
                            uid_parent = ".intval($uid_parent).",
                            level      = ".intval($node->getTree()->level->value())."
                        WHERE uid = ".intval($node->uid->value()); 
                $this->_con->exec($sql);    
            }
            $sql = "DELETE FROM si_tree WHERE ispage = 0 AND uid_parent = ".$parentRand;
            $this->_con->exec($sql);
            
            $this->_con->commit();    
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            die();
            return false;
        } 
        return true;
    }
    
    public function getMaxChilds($parent, $isContent, $isDraft = false)
    {
        $pageSql = $isContent ? 't.ispage = 0' : 't.ispage = 1';
        
        if($isDraft){
            $subQuery2 = self::getQueryStringVersionNodes(); 
            $sql = "SELECT COUNT(*)  
                    FROM si_tree AS t
                    INNER JOIN si_nodes AS n
                       ON t.uid = n.uid
                    WHERE t.uid_parent = $parent
                          AND $pageSql
                          AND ($subQuery2)";
        }else{
            $sql = "SELECT COUNT(*) 
                    FROM si_tree AS t
                    INNER JOIN si_nodes AS n
                       ON t.uid = n.uid
                    WHERE t.uid_parent = $parent
                          AND $pageSql
                          AND n.version = 0";
        }
        
        return $this->_con->fetchOne($sql);
    }
    
    public function movedown(Sicore_Models_Node $node, $isContent = true)
    {
        if(!$node->getParent()) return false;
        
        $parent          = $node->getTree()->parent->value(); 
        $uid             = $node->uid->value(); 
        $currentPosition = $node->getTree()->sorting->value();
        $newPosition     = $currentPosition + 1;
        $maxChilds       = $this->getMaxChilds($node->getParent()->uid->value(), $isContent, true);
        if($currentPosition >= $maxChilds) return false;
        
        $pageSql = $isContent ? 'ispage = 0' : 'ispage = 1'; 
        
        $this->_con->beginTransaction(); 
        try{
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = -1 
                    WHERE $pageSql
                          AND uid_parent = ".intval($parent)."
                          AND sorting = ".intval($newPosition);
            $this->_con->exec($sql); 
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = ".intval($newPosition)." 
                    WHERE $pageSql
                          AND uid = ".intval($uid);
            $this->_con->exec($sql); 
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = ".intval($currentPosition)." 
                    WHERE $pageSql
                          AND uid_parent = ".intval($parent)."
                          AND sorting = -1";
            $this->_con->exec($sql); 
            
            $this->_con->commit();
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            return false;
        }
        
        return true;
    }
    
    public function moveup(Sicore_Models_Node $node, $isContent = true)
    {
        if(!$node->getParent()) return false;
        
        $parent          = $node->getTree()->parent->value(); 
        $uid             = $node->uid->value(); 
        $currentPosition = $node->getTree()->sorting->value();
        $newPosition     = $currentPosition - 1;
        if($newPosition == 0) return false;
        
        $pageSql = $isContent ? 'ispage = 0' : 'ispage = 1'; 
        
        $this->_con->beginTransaction(); 
        try{
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = -1 
                    WHERE $pageSql 
                          AND uid_parent = ".intval($parent)."
                          AND sorting = ".intval($newPosition);
            $this->_con->exec($sql); 
            
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = ".intval($newPosition)." 
                    WHERE $pageSql
                          AND uid = ".intval($uid);
            
            $this->_con->exec($sql); 
            //Knoten verschieben
            $sql = "UPDATE si_tree 
                    SET sorting = ".intval($currentPosition)." 
                    WHERE $pageSql
                          AND uid_parent = ".intval($parent)."
                          AND sorting = -1";
             
            $this->_con->exec($sql); 
            
            
            $this->_con->commit();
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            return false;
        }
        
        return true;
    }
    
    /**
     * Verschiebt einen Knoten
     *
     * @param Sicore_Models_Node $nodeSource
     * @param Sicore_Models_Node $nodeDest   
     * @param String $type 'before', 'after' oder 'inside
     */
    public function move(Sicore_Models_Node $nodeSource, Sicore_Models_Node $nodeDest, $type)
    {
        
        if(empty($type)){
            return false;
        }
        
        $this->_con->beginTransaction();
        try{
            //Level-Offset   
            $newLevel    = $type == 'inside' ? $nodeDest->getTree()->level->value()+1 : $nodeDest->getTree()->level->value();
            $levelOffset = $newLevel - $nodeSource->getTree()->level->value();
            //Knoten wird in gleicher Ebene verschoben
            if(($nodeSource->getTree()->parent->value() == $nodeDest->getTree()->parent->value() && $type != 'inside') || ($nodeSource->getTree()->parent->value() == $nodeDest->uid->value() && $type == 'inside')){
                $sameTree   = true;
                $parent     = $nodeDest->getTree()->parent->value(); 
                $parentDest = $nodeDest->getTree()->parent->value();   
                
                $parentRand = -rand(1,10000);
                $sql = "UPDATE si_tree
                        SET sorting = -1,
                            uid_parent = ".$parentRand."
                        WHERE uid=".intval($nodeSource->uid->value());
                $this->_con->exec($sql);
                
                //Knoten wird nach oben verschoben
                if($nodeSource->getTree()->sorting->value() > $nodeDest->getTree()->sorting->value() || $nodeSource->getTree()->parent->value() == $nodeDest->uid->value()){
                    switch($type){
                        case 'after':
                            $sortingNew = $nodeDest->getTree()->sorting->value()+1;
                            break;
                        case 'before':
                            $sortingNew = $nodeDest->getTree()->sorting->value();
                            break; 
                        default:
                            return false;
                    }   
                    //Reihenfolge anpassen
                    $sql = "UPDATE si_tree
                            SET sorting = sorting + 1
                            WHERE ispage = 1 
                                  AND uid_parent = ".intval($parent)."
                                  AND sorting < ".intval($nodeSource->getTree()->sorting->value())."
                                  AND sorting >= ".intval($sortingNew)."
                            ORDER BY sorting DESC";
                     $this->_con->exec($sql);
                //Knoten wird nach unten verschoben  
                }else{
                     switch($type){
                        case 'after':
                            $sortingNew = $nodeDest->getTree()->sorting->value();
                            break;
                        case 'before':
                            $sortingNew = $nodeDest->getTree()->sorting->value()-1;
                            break;  
                        default:
                            return false;
                    }   
                    //Reihenfolge anpassen
                     $sql = "UPDATE si_tree
                            SET sorting = sorting - 1
                            WHERE ispage = 1 
                                  AND uid_parent = ".intval($parent)."
                                  AND sorting > ".intval($nodeSource->getTree()->sorting->value())."
                                  AND sorting <= ".intval($sortingNew)."
                            ORDER BY sorting ASC";
                     $this->_con->exec($sql);
                }
                
            //Knoten wird in andere Ebene verschoben    
            }else{      
                $sameTree = false; 
                switch($type){
                    case 'after':
                        $sortingNew = $nodeDest->getTree()->sorting->value()+1;
                        $parentDest = $nodeDest->getTree()->parent->value();  
                        break;
                    case 'before':
                        $sortingNew = $nodeDest->getTree()->sorting->value();
                        $parentDest = $nodeDest->getTree()->parent->value(); 
                        break; 
                    case 'inside':
                        $sortingNew = 1;
                        $parentDest = $nodeDest->uid->value();  
                        break;    
                    default:
                        return false;
                }    
                $parentRand = -rand(1,10000);
                $sql = "UPDATE si_tree
                        SET sorting = -1,
                            uid_parent = ".$parentRand."
                        WHERE uid=".intval($nodeSource->uid->value());
                $this->_con->exec($sql);
                $this->_con->exec($sql);
                //Reihenfolge anpassen
                $sql = "UPDATE si_tree
                        SET sorting = sorting - 1
                        WHERE ispage = 1 
                              AND uid_parent = ".intval($nodeSource->getTree()->parent->value())."
                              AND sorting > ".intval($nodeSource->getTree()->sorting->value())."
                        ORDER BY sorting ASC";
                $this->_con->exec($sql);
                $sql = "UPDATE si_tree
                        SET sorting = sorting + 1
                        WHERE ispage = 1 
                              AND uid_parent = ".intval($parentDest)."
                              AND sorting >= ".intval($sortingNew)."
                        ORDER BY sorting DESC";
                $this->_con->exec($sql);              
                //si_urlalias anpassen
                $urlaliasCurrent = $nodeSource->getUrlalias()->alias->value();
                if($type == 'inside'){
                    $urlaliasRoot = $nodeDest->getUrlalias()->alias->value();     
                }elseif(($level = $nodeDest->getTree()->level->value())){
                    $parentNode   = $nodeDest->getParent($level - 1);
                    $urlaliasRoot = $parentNode->getUrlalias()->alias->value();
                }else{
                    $urlaliasRoot = '';
                }
                
                $mUrlalias = new Sicore_Models_Server_Db_Urlalias();
                if($urlaliasCurrent && ($urlalias = $mUrlalias->check($urlaliasRoot, $nodeSource->title->value(), $nodeSource->uid->value()))){
                    $urlalias            = $this->_con->quote($urlalias);
                    $urlaliasCurrentSQL1 = $this->_con->quote($urlaliasCurrent);
                    $urlaliasCurrentSQL2 = $this->_con->quote($urlaliasCurrent.'%'); 
            
                    $sql = "INSERT INTO si_urlalias
                              (alias, nodes_uid, language_code, version)
                            SELECT alias, nodes_uid, language_code, -1 
                            FROM  si_urlalias 
                            WHERE alias LIKE $urlaliasCurrentSQL2  
                            ON DUPLICATE KEY UPDATE
                            version = -1";
                    
                    $this->_con->exec($sql);    
                    $sql = "UPDATE si_urlalias
                            SET alias =  REPLACE(alias, $urlaliasCurrentSQL1, $urlalias)
                            WHERE alias LIKE $urlaliasCurrentSQL2";
                    $this->_con->exec($sql);     
                }                 
            } 
            
            //Knoten anpassen
            $sql = "UPDATE si_tree 
                    SET uid_parent = ".intval($parentDest).", 
                        sorting =  ".intval($sortingNew).",
                        level  = ".intval($newLevel)."
                    WHERE uid = ".intval($nodeSource->uid->value());
           
            $this->_con->exec($sql);
            
            //si_tree_tree ergänzen
            if(!$sameTree){
                $sql = "DELETE FROM si_tree_tree 
                        WHERE tree_uid_child = ".intval($nodeSource->uid->value());
                $this->_con->exec($sql); 
                
                $tableName = 'si_temp'.rand();
                $sql = "CREATE TABLE $tableName ENGINE=MEMORY SELECT * FROM si_tree_tree WHERE 1=0";
                $this->_con->exec($sql);
                
                $rootline = $nodeDest->getAllParents();
                if($type == 'inside'){
                    $sql = "INSERT INTO $tableName
                                (tree_uid_child, tree_uid_parent)
                             VALUES(".$this->_con->quote($nodeSource->uid->value()).", ".$this->_con->quote($nodeDest->uid->value()).")";
                     $this->_con->exec($sql);  
                     $sql = "INSERT INTO $tableName 
                               (tree_uid_child, tree_uid_parent)
                             SELECT tree_uid_child,
                                    ".$this->_con->quote($nodeDest->uid->value())."
                             FROM si_tree_tree  
                             WHERE tree_uid_parent = ".$this->_con->quote($nodeSource->uid->value());
                     $this->_con->exec($sql); 
                }   
                foreach($rootline as $node){
                     $sql = "INSERT $tableName
                                (tree_uid_child, tree_uid_parent)
                             VALUES(".$this->_con->quote($nodeSource->uid->value()).", ".$this->_con->quote($node->uid->value()).")";
                     
                     $this->_con->exec($sql);                      
                     $sql = "INSERT INTO $tableName 
                               (tree_uid_child, tree_uid_parent)
                             SELECT tree_uid_child,
                                    ".$this->_con->quote($node->uid->value())."
                             FROM si_tree_tree  
                             WHERE tree_uid_parent = ".$this->_con->quote($nodeSource->uid->value());
                     $this->_con->exec($sql); 
                 } 
                 $sql = "INSERT INTO $tableName 
                           (tree_uid_child, tree_uid_parent)
                         SELECT tree_uid_child,
                                ".$this->_con->quote($nodeSource->uid->value())."
                         FROM si_tree_tree  
                         WHERE tree_uid_parent = ".$this->_con->quote($nodeSource->uid->value());
                 $this->_con->exec($sql);
                
                $sql = "DELETE FROM si_tree_tree 
                        WHERE tree_uid_child IN(SELECT tree_uid_child FROM $tableName WHERE tree_uid_parent = ".intval($nodeSource->uid->value()).")";
                $this->_con->exec($sql);
                
                $sql = "INSERT INTO si_tree_tree
                        SELECT * FROM $tableName";
                $this->_con->exec($sql); 
                 
                $sql = "DROP TABLE $tableName";
                $this->_con->exec($sql);
            } 
            $this->_con->commit();
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            die();
            return false;
        }
        
        return true;
    }
    
    /**
     * Veröffentlicht einen Knoten (öffentlicher Zugriff)  
     *
     * @param Sicore_Models_Node_Page $node    
     */
    public function publish(Sicore_Models_Node_Page $node, $withoutContent = false, $withSubpages = false, $transactionStarted = false)
    {
        if(!$transactionStarted) $this->_con->beginTransaction(); 
        try{
            $this->_doPublish($node);
            $cache = Sicore_Models_Registry::getCache();
            $cache->clean(
                Zend_Cache::CLEANING_MODE_MATCHING_TAG,
                array('content', (string)($node->uid->value()))
            );
            if(!$withoutContent){
                $childs = $this->doCheckIsHidden(false)
                               ->doCheckIsDeleted(false)
                               ->doCheckIsActive(false)
                               ->doCheckDate(false)
                               ->getByParentUid($node->uid->value(), array('Sicore_Models_Node_Content'), array(), true, 0, true);
                if($childs){  
                    krsort($childs); 
                    foreach($childs as $child){
                        $this->_doPublish($child); 
                    } 
                }
            }
            if($withSubpages){
                $childs = $this->doCheckIsHidden(false)
                               ->doCheckIsDeleted(false)
                               ->doCheckIsActive(false)
                               ->doCheckDate(false)
                               ->getByParentUid($node->uid->value(), array('Sicore_Models_Node_Page'), array(), true, 0, true);
                if($childs){
                    krsort($childs); 
                    foreach($childs as $child){
                        $this->publish($child, $withoutContent, true, true); 
                    } 
                }    
            }
            if(!$transactionStarted) $this->_con->commit();   
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            return false;
        }
    }
    
    /**
     * Veröffentlicht einen Knoten (interne Verarbeitung)
     *
     * @param Sicore_Models_Node $node    
     */
    protected function _doPublish(Sicore_Models_Node $node){
        
        if($node->version_status->value() == 'DELETED'){
            //Subnodes löschen
            $tableName = 'si_temp'.rand();
            $sql = "CREATE TABLE $tableName ENGINE=MEMORY 
                    SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value());
            $this->_con->exec($sql);
            $sql = "DELETE FROM si_tree
                    WHERE uid = ".intval($node->uid->value())."
                          OR uid IN (SELECT tree_uid_child FROM $tableName)";
            $this->_con->exec($sql);  
            $sql = "DROP TABLE $tableName";
            $this->_con->exec($sql); 
            //Positionierung anpassen
            $sql = "UPDATE si_tree
                    SET sorting = sorting - 1
                    WHERE uid_parent = ".intval($node->getTree()->parent->value())."
                          AND sorting > ".intval($node->getTree()->sorting->value())." 
                          AND ispage = ".intval($node->getTree()->isPage->value())."
                    ORDER BY sorting ASC";
            $this->_con->exec($sql);
        }elseif($node->version_status->value()){
            //Node veröffentlichen
            $sql = "DELETE FROM si_nodes 
                    WHERE uid = ".intval($node->uid->value())."
                          AND version = 0";
            $this->_con->exec($sql);
            
            $sql = "UPDATE si_nodes 
                    SET version = 0, publishfield = NULL, publishdeletesubfields = NULL, version_status = NULL 
                    WHERE uid = ".intval($node->uid->value())."
                          AND version = 1";
            $this->_con->exec($sql);
            //Urlalias veröffentlichen            
            if($node->getUrlalias()->version->value() > 0){
                $sql = "UPDATE IGNORE si_urlalias 
                        SET version = -1
                        WHERE (nodes_uid = ".intval($node->uid->value())."
                               OR nodes_uid IN(SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value())." )
                              )
                              AND version = 0";
                $this->_con->exec($sql);
                $sql = "DELETE FROM si_urlalias 
                        WHERE (nodes_uid = ".intval($node->uid->value())."
                               OR nodes_uid IN(SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value())." )
                              )
                              AND version = 0";
                $this->_con->exec($sql);
                $sql = "UPDATE si_urlalias 
                        SET version = 0
                        WHERE (nodes_uid = ".intval($node->uid->value())."
                               OR nodes_uid IN(SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value())." )
                              )
                              AND version = 1";
                $this->_con->exec($sql);
            }
        }
    }
    
    /**
     * Setzt einen Knoten zurück (öffentlicher Zugriff)  
     *
     * @param Sicore_Models_Node_Page $node    
     */
    public function undo(Sicore_Models_Node_Page $node, $withoutChilds = false)
    {
        $this->_con->beginTransaction(); 
        try{
            $this->_doUndo($node);
            if(!$withoutChilds){
                $childs = $this->doCheckIsHidden(false)
                               ->doCheckIsDeleted(false)
                               ->doCheckIsActive(false)
                               ->doCheckDate(false)
                               ->getByParentUid($node->uid->value(), array('Sicore_Models_Node_Content'), array(), true);
                krsort($childs);   
                foreach($childs as $child){
                    $this->_doUndo($child); 
                } 
            }
            $this->_con->commit();   
        }catch(Exception $e){
            $this->_con->rollback();
            Sicore_Models_Logger::exception($e); 
            die();
            return false;
        }
    }
    
    /**
     * Setzt einen Knoten zurück (öffentlicher Zugriff)  
     *
     * @param Sicore_Models_Node_Page $node    
     */
    protected function _doUndo(Sicore_Models_Node $node)
    {
        if($node->version_status->value() == 'ADDED'){
            //Subnodes löschen
            $tableName = 'si_temp'.rand();
            $sql = "CREATE TABLE $tableName ENGINE=MEMORY 
                    SELECT tree_uid_child FROM si_tree_tree WHERE tree_uid_parent = ".intval($node->uid->value());
            $this->_con->exec($sql);
            
            $sql = "DELETE FROM si_tree
                    WHERE uid = ".intval($node->uid->value())."
                          OR uid IN (SELECT tree_uid_child FROM $tableName)";
            $this->_con->exec($sql);
            $sql = "DROP TABLE $tableName";
            $this->_con->exec($sql); 
            //Positionierung anpassen
            $sql = "UPDATE si_tree
                    SET sorting = sorting - 1
                    WHERE uid_parent = ".intval($node->getTree()->parent->value())."
                          AND sorting > ".intval($node->getTree()->sorting->value())."
                          AND ispage = ".intval($node->getTree()->isPage->value())." 
                    ORDER BY sorting ASC";
            $this->_con->exec($sql);    
        }else{
            //Node zurücksetzen
            $sql = "DELETE FROM si_nodes
                    WHERE uid = ".intval($node->uid->value())."
                          AND version = 1";
            $this->_con->exec($sql);
            //Urlalias zurücksetzen
            $sqlAlias = $this->_con->quote($node->getUrlalias()->alias->value().'%');  
            $sql = "DELETE FROM si_urlalias
                    WHERE alias LIKE $sqlAlias 
                          AND version = 1";
            $this->_con->exec($sql);                              
        }
    }
    
    /**
     * Fügt einen neuen Knoten ein
     *
     * @param Sicore_Models_Node $node    
     */
    protected function insert(Sicore_Models_Node $node, $isAdded = true)
    {
        $data = array();
        foreach($node as $property){
            if(!($property instanceof Sicore_Models_Node_Property_Database_Interface) || $property->doNotSave()) continue;
            $data = $property->toDatabase($data);
        }
        if($isAdded) $data['added']   = new Zend_Db_Expr('NOW()');
        $data['changed'] = new Zend_Db_Expr('NOW()'); 
        
        $this->_con->insert('si_nodes', $data);
    }
    
    
    public static function getQueryStringSubnodes()
    {      
        $subQuery = "SELECT MAX(n_s1.version)  
                     FROM si_nodes AS n_s1
                     INNER JOIN si_tree AS t_s1
                       ON n_s1.uid = t_s1.uid         
                     WHERE t_s1.uid_parent = t.uid AND t_s1.ispage = 0";
        return $subQuery;
    }
    
    public static function getQueryStringVersionNodes()
    {      
        $subQuery = "SELECT MAX(version) 
                     FROM si_nodes AS n_s2 
                     WHERE n_s2.uid = n.uid LIMIT 1";
        return $subQuery;
    }
    
    public static function getQueryStringVersionUrlalias()
    {      
        $subQuery = "SELECT MAX(version) 
                     FROM si_urlalias AS url_s3 
                     WHERE url_s3.nodes_uid = url.nodes_uid LIMIT 1";
        return $subQuery;
    }
    
    protected function _getQueryStringUidParent($uid) 
    {
        $uid = intval($uid);
        
        if($uid == 0){
            $sqlUidparent = '(t.uid_parent IS NULL OR t.uid_parent = 0)';
        }else{
            $sqlUidparent = " t.uid_parent = ".$uid;
        }    
          
        return $sqlUidparent;
    }
    
}
?>
