<?php
/**
 * SIMPLOO CMS
 * Das Simploo CMS ist für Privatpersonen, Vereine und 
 * sonstigen Organisationen ohne kommerziellen Hintergrund 
 * in der SIMPLOO CMS COMMUNITY EDITION kostenlos einsetzbar.
 * 
 * Gewerbliche Kunden mit kommerziellem Hintergrund 
 * benötigen die SIMPLOO CMS BUSINESS EDITION.
 * 
 * In beiden Fällen müssen Sie über www.simploo.de 
 * eine Seriennummer registrieren und in der Software 
 * nach Anleitung entsprechend aktivieren.
 *
 * @category   Simploo
 * @package    Sicore_Models
 * @copyright  Copyright (c) 2009 Simploo GmbH (http://www.simploo.de)
 * @license    http://www.simploo.de
 * @version    $Id: Uobj.php 1241 2010-03-15 19:33:52Z ms $
 */
abstract class Sicore_Models_Uobj extends Sicore_Models_Database
{
    
    
    
    /**
     * Übergeordnete Gruppen
     *
     * @var Array
     */
protected $_parentGroups=array();
    
    /**
     * Konfiguration
     *
     * @var String
     */
    protected $_config = null;
   
     
    /**
     * Konstruktor
     *
     * @param Array $rawData Datenbankfelder des Datensatzes
     */
    public function __construct($rawData = null){
        $this->_config = Sicore_Models_Registry::getConfig(Sicore_Models_Registry::CONFIG_USERS);
        $this->_loadConfig($this->_config->Sicore_Models_Uobj);
      
        $this->init($rawData); 
    } 
    
       
    /**
     * Gibt die interne ID des Elternobjektes zurück
     *
     * @return Integer
     */
    public function getUids()
    {
        $data[] = $this->uid->value();
        foreach($this->_parentGroups as $parent){
            $data[] = $parent->uid->value();           
        }
        return $data;
    }
    

    /**
     * Gibt alle Elterngruppen zurück
     *
     * @param Boolean Umgekehrte Reihenfolge
     * @return Array
     */
    public function getAllParentGroups($desc = false)
    {
        if($desc){
            return array_reverse($this->_parentGroups);
        }else{
            return $this->_parentGroups;
        }   
    }
        
    /**
     * Gibt die kombinierte GUI-Menüliste zurück
     *
     * @return Array
     */
    public function getGuiMenu()
    {
        return array_merge($this->gui_menu_enabled->toArray(), $this->gui_menu_readonly->toArray()); 
    }  
     
    
    /**
     * Überprüft, ob das Objekt einer bestimmten Gruppe angeört
     * Übergeben werden kann 
     * - eine Instanz der Klasse Sicore_Models_Uobj_Group
     * - die UID der zu überprüfenden Gruppe
     * - der Alias der zu überprüfenden Klasse
     *
     * @param Sicore_Models_Uobj_Group / Integer / String $ingroup
     * @return Boolean
     */
    public function isInGroup($ingroup)
    {
        foreach($this->_parentGroups as $group){
            if($ingroup instanceof Sicore_Models_Uobj_Group){
                if($ingroup->uid->value() == $group->uid->value()){
                    return true;
                }
            }elseif(is_int($ingroup)){
                if($group->uid->value() == $ingroup){
                    return true;
                }
            }elseif(is_string($ingroup)){
                if($group->alias->value() == $ingroup){
                    return true;
                }
            }
        }
        
        return false;
    }
    
    /**
     * Fügt eine Elterngruppe hinzu
     *
     * @param $group Sicore_Models_Uobj_Group
     * @return Sicore_Models_Uobj
     */
    public function addParentGroup(Sicore_Models_Uobj_Group $group)
    {
        $this->_parentGroups[] = $group;
        if(!$group->isactive->value()){
            $this->isactive->set(false);
        }
        if($group->isadmin->value()){
            $this->isadmin->set(true);
        }
        if($group->gui_showpanel->value()){
            $this->gui_showpanel->set(true);
        }

        if(!$this->gui_menu_enabled->value()) $this->gui_menu_enabled->set($group->gui_menu_enabled->value());
        if(!$this->gui_menu_readonly->value()) $this->gui_menu_readonly->set($group->gui_menu_readonly->value());
        
        return $this;
    }   
    
      
}

